      PROGRAM RMRPP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH MULTI-RESPONSE PERMUTATION
C         PROCEDURES (MRPP) IS CALCULATED IN THIS FORTRAN-77 PROGRAM.
C         THIS TEST IS DESIGNED TO ANALYZE MULTIVARIATE DATA AT THE ORDINAL
C         (OR HIGHER) LEVELS AND IS BASED ON THE AVERAGE STANDARDIZED
C         EUCLIDEAN DISTANCE BETWEEN OBJECT POSITIONS WITHIN A PRIORI
C         DISJOINT GROUPS OF OBJECTS IN A FINITE POPULATION WHERE EACH
C         POSITION IS A POINT IN AN R-DIMENSIONAL SPACE (THUS THE ARGUMENTS
C         OF EACH R-DIMENSIONAL POINT CORRESPOND TO THE R MEASURED RESPONSES
C         OF EACH OBJECT IN THE FINITE POPULATION OF OBJECTS).  THIS PROGRAM
C         ALSO CONTAINS THE TRUNCATED DISTANCE OPTION (THE UPPER TRUNCATION
C         CONSTANT IS TERMED SUP).  THE PRESENT MAXIMUM VALUES OF G, N AND
C         R IN THIS PROGRAM ARE 40, 1800 AND 50, RESPECTIVELY.  THIS PROGRAM
C         CAN INVOKE EITHER AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.)
C         COMMENSURATION ON EACH RESPONSE OR A HOTELLING-TYPE COMMENSURATION.
C         THE A(N,H), B(N,H) AND C(N,H) RANKS TEST ARE AN ADDITIONAL OPTION.
C
C         PROGRAM MODIFIED 12/15/2007
C
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = n(I)/sum(n(I))
C            CFORM = 2:  C(I) = (n(I)-1)/sum(n(I)-1)
C            CFORM = 3:  C(I) = 1/sum(1)
C            CFORM = 4:  C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))
C       CFORM IS PASSED AS THE FOURTH PARAMETER ON THE SECOND RECORD
C       IN THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED IN
C       SUBROUTINE DIST, AND STORED IN ARRAY C(I).
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION U(17),ITHGRP(40),XI(40),AD(50),DATA(1800,51),RD(1800,50)
      INTEGER   CFORM
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      READ(3,100) HEAD
      READ(3,*) NCASES,ID,V,SUP,LZ,IR,KT,H,IC,CFORM,NGRPS
      READ(3,*) (ITHGRP(I),I = 1,NGRPS)
      L = 1
      M = ITHGRP(L)
      DO 160 I = 1,NCASES
         READ(3,*,END=170) (DATA(I,J),J = 1,ID)
  140    IF(I.LE.M) GO TO 150
         L = L+1
         IF(L.LE.NGRPS) THEN
            M = M+ITHGRP(L)
         ELSE
            M = NCASES
         END IF
         GO TO 140
  150    DATA(I,ID+1) = ONE*L
  160 CONTINUE
      IF(ID.EQ.1) LZ = 0
      GO TO 180
  170 WRITE(*,175) 'ERROR:  END-OF FILE ENCOUNTERED'
  175 FORMAT(/,T3,A)
      STOP
  180 KZ = 0
      DO 200 I = 1,NGRPS
         KZ = KZ+ITHGRP(I)
  200 CONTINUE
      IF(KZ.GT.NCASES) STOP 'SUM OF GROUP SIZES EXCEEDS TOTAL CASES'
      IF(IR.NE.1) GO TO 1080
      DO 270 J = 1,ID
         CL = 1.0D30
         DO 210 I = 1,NCASES
            IF(DATA(I,J).LT.CL) CL = DATA(I,J)
  210    CONTINUE
         DO 220 I = 1,NCASES
            DATA(I,J) = DATA(I,J)-CL+1
  220    CONTINUE
         PHI = ONE+1.0D-12
         A1 = ZERO
         A2 = ZERO
         A3 = ONE*NCASES-0.1D0
         B1 = ZERO-1.0D30
         B2 = 1.0D30
  230    IF(A2.GT.A3) GO TO 270
         DO 240 I = 1,NCASES
            IF(DATA(I,J).GT.B1.AND.DATA(I,J).LT.B2) B2 = DATA(I,J)*PHI
  240    CONTINUE
         DO 250 I = 1,NCASES
            W = DABS(ONE-DATA(I,J)/B2)
            IF(W.LT.1.0D-11) A1 = A1+1
  250    CONTINUE
         A4 = A2+(A1+1)/2
         DO 260 I = 1,NCASES
            W = DABS(ONE-DATA(I,J)/B2)
            IF(W.LT.1.0D-11) RD(I,J) = A4
  260    CONTINUE
         A2 = A2+A1
         A1 = ZERO
         B1 = B2
         B2 = 1.0D30
         GO TO 230
  270 CONTINUE
      Z = ONE*(NCASES+1)/2
      DO 290 I = 1,NCASES
         DO 280 J = 1,ID
            GO TO (272,274,276) KT
  272       DATA(I,J) = RD(I,J)**H
            GO TO 280
  274       DATA(I,J) = ZERO
            W = DABS(Z-RD(I,J))
            IF(W.LT.1.0D-11) GO TO 280
            DATA(I,J) = DABS(RD(I,J)-Z)**H
            GO TO 280
  276       DATA(I,J) = ZERO
            W = DABS(Z-RD(I,J))
            IF(W.LT.1.0D-11) GO TO 280
            IF(RD(I,J).GT.Z) THEN
               DATA(I,J) = (RD(I,J)-Z)**H
            ELSEIF(RD(I,J).LT.Z) THEN
               DATA(I,J) = ZERO-(Z-RD(I,J))**H
            ENDIF
  280    CONTINUE
  290 CONTINUE
 1080 IF(ID.EQ.1.OR.IR.EQ.1.OR.LZ.EQ.1.OR.IC.NE.1) GO TO 1097
      DO 1090 I = 1,ID
         AD(I) = ZERO
         DO 1088 J = 1,NCASES-1
            DO 1086 K = J+1,NCASES
               AD(I) = AD(I)+(DABS(DATA(J,I)-DATA(K,I)))**V
 1086       CONTINUE
 1088    CONTINUE
         AD(I) = AD(I)**(ONE/V)
 1090 CONTINUE
      DO 1095 I = 1,NCASES
         DO 1093 J = 1,ID
            DATA(I,J) = DATA(I,J)/AD(J)
 1093    CONTINUE
 1095 CONTINUE
 1097 CALL DIST(NCASES,ID,V,SUP,CFORM,NGRPS,U,ITHGRP,DATA,XI,
     +   DBAR,PL,PU,LZ,MS)
      WRITE(6,100) HEAD
      IF(LZ.EQ.1) THEN
         HEAD = 'MULTIVARIATE HOTELLING-TYPE PERMUTATION TESTS'
      ELSE
         HEAD = 'MULTI-RESPONSE PERMUTATION PROCEDURES'
      END IF
      WRITE(6,1100) HEAD
 1100 FORMAT(T23,A,/)
      WRITE(6,1200) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1200 FORMAT(T10,A,I4,A)
      WRITE(6,1200) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1300) 'IN: ', NGRPS, ' GROUPS OF SIZES ',
     +              (ITHGRP(I),I = 1,NGRPS)
 1300 FORMAT(T10,A,I2,A,10I5)
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = n(I)/sum(n(I))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = (n(I)-1)/sum(n(I)-1)'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = 1/sum(1)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))'
 1450 FORMAT(T10,A)
 1500 WRITE(6,100) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZE ', ITHGRP(I),
     +                 ' HAS AN AVERAGE DISTANCE =', XI(I)
 1600    FORMAT(T10,A,I2,A,I4,A,G16.8)
 1700 CONTINUE
      WRITE(6,100) ' '
      WRITE(6,1710) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
      WRITE(6,1710) 'NUMBER OF SIMULATIONS = ',MS
 1710 FORMAT(T10,A,I9)
      IF(IR.EQ.1) THEN
         GO TO (1725,1750,1775) KT
 1725    HEAD = 'MRPP VERSION OF A(N,H) RANKS TEST'
         WRITE(6,1790) HEAD
         WRITE(6,1800) 'A(N,H) RANKS TEST EXPONENT H = ',H
         GO TO 1810
 1750    HEAD = 'MRPP VERSION OF B(N,H) RANKS TEST'
         WRITE(6,1790) HEAD
         WRITE(6,1800) 'B(N,H) RANKS TEST EXPONENT H = ',H
         GO TO 1810
 1775    HEAD = 'MRPP VERSION OF C(N,H) RANKS TEST'
         WRITE(6,1790) HEAD
         WRITE(6,1800) 'C(N,H) RANKS TEST EXPONENT H = ',H
      END IF
 1790 FORMAT(T10,A)
 1800 FORMAT(T10,A,G16.8)
 1810 IF(LZ.EQ.0) THEN
         WRITE(6,1710) '1 (0) IMPLIES (NO) AVE. D.F. COMMENSURATION:',IC
      END IF
      WRITE(6,1800) 'DISTANCE EXPONENT =', V
      WRITE(6,1800) 'UPPER DISTANCE BOUND =', SUP
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
      WRITE(6,1800) 'LOWER TAIL P-VALUE OF OBSERVED DELTA =', PL
      WRITE(6,1800) 'UPPER TAIL P-VALUE OF OBSERVED DELTA =', PU
      WRITE(6,1850)
 1850 FORMAT('********************************************************')
      CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE DIST(NCASES,ID,V,SUP,CFORM,NGRPS,U,ITHGRP,DATA,XI1,
     +   DBAR,PL,PU,LZ,MS)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  U(17),C(40),DATA(1800,51),DEL(1800,1800),ITHGRP(40),
     +           XI1(40),XI(40),A(50),Y(50,50),KP(50),KR(50),KC(50)
      INTEGER  CFORM
C
      ND = 50
      ZERO = 0.0D0
      ONE = 1.0D0
      DBAR = ZERO
      DO 10 I = 1,NGRPS
         XI1(I) = ZERO
   10 CONTINUE
      IF(LZ.EQ.1) GO TO 50
      DO 40 I = 1,NCASES
         DO 30 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 20 K = 1,ID
               DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
   20       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
            IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
            IS = IDNINT(DATA(I,ID+1))
            JS = IDNINT(DATA(J,ID+1))
            IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI1(IS) = XI1(IS)+DEL(I,J)
   30    CONTINUE
   40 CONTINUE
      GO TO 1405
   50 DO 200 I = 1,ID
         A(I) = ZERO
         DO 100 J = 1,NCASES
            A(I) = A(I)+DATA(J,I)
  100    CONTINUE
         A(I) = A(I)/NCASES
  200 CONTINUE
      DO 500 I = 1,ID
         DO 400 J = I,ID
            Y(I,J) = ZERO
            DO 300 K = 1,NCASES
               Y(I,J) = Y(I,J)+(DATA(K,I)-A(I))*(DATA(K,J)-A(J))
  300       CONTINUE
            Y(J,I) = Y(I,J)
  400    CONTINUE
  500 CONTINUE
      CALL INVERT(ID,Y,KP,KR,KC,ND)
      DO 1400 I = 1,NCASES
         DO 1300 J = 1,NCASES
            DEL(I,J) = ZERO
            DO 1200 K = 1,ID
               DO 1100 L = 1,ID
                  DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))*Y(K,L)*
     1                                (DATA(I,L)-DATA(J,L))
 1100          CONTINUE
 1200       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
            IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
            IS = IDNINT(DATA(I,ID+1))
            JS = IDNINT(DATA(J,ID+1))
            IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI1(IS) = XI1(IS)+DEL(I,J)
 1300    CONTINUE
 1400 CONTINUE
 1405 GO TO (1410,1430,1450,1470) CFORM
 1410 FK = ZERO
      DO 1415 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1415 CONTINUE
      DO 1420 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)/FK
 1420 CONTINUE
      GO TO 1490
 1430 FK = ZERO
      DO 1435 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1435 CONTINUE
      DO 1440 I = 1,NGRPS
         C(I) = ONE*(ITHGRP(I)-1)/(FK-NGRPS)
 1440 CONTINUE
      GO TO 1490
 1450 DO 1460 I = 1,NGRPS
         C(I) = ONE/NGRPS
 1460 CONTINUE
      GO TO 1490
 1470 W = ZERO
      DO 1475 I = 1,NGRPS
         W = W+ONE*ITHGRP(I)*(ITHGRP(I)-1)
 1475 CONTINUE
      DO 1480 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)*(ITHGRP(I)-1)/W
 1480 CONTINUE
 1490 DO 1500 I = 1,NGRPS
         XI1(I) = XI1(I)/(ITHGRP(I)*(ITHGRP(I)-1))
         DBAR = DBAR+C(I)*XI1(I)
         DL = DBAR*(ONE+1.0D-12)
         DU = DBAR*(ONE-1.0D-12)
 1500 CONTINUE
      ML = 0
      MU = 0
      DO 3600 IW = 1,MS
         CALL SAMPWOR(NCASES,NCASES,ID,U,DATA)
         L = 1
         MA = ITHGRP(L)
         DO 2006 I = 1,NCASES
 2004    IF(I.LE.MA) GO TO 2005
            L = L+1
            IF(L.LE.NGRPS) THEN
               MA = MA+ITHGRP(L)
            ELSE
               MA = NCASES
            END IF
            GO TO 2004
 2005       DATA(I,ID+1) = ONE*L
 2006    CONTINUE
         ND = 50
         DZ = ZERO
         DO 2010 I = 1,NGRPS
            XI(I) = ZERO
 2010    CONTINUE
         IF(LZ.EQ.1) GO TO 2050
         DO 2040 I = 1,NCASES
            DO 2030 J = 1,NCASES
               DEL(I,J) = ZERO
               DO 2020 K = 1,ID
                  DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
 2020          CONTINUE
               DEL(I,J) = DEL(I,J)**(V/2)
               IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
               IS = IDNINT(DATA(I,ID+1))
               JS = IDNINT(DATA(J,ID+1))
               IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI(IS) = XI(IS)+DEL(I,J)
 2030       CONTINUE
 2040    CONTINUE
         GO TO 3405
 2050    DO 2200 I = 1,ID
            A(I) = ZERO
            DO 2100 J = 1,NCASES
               A(I) = A(I)+DATA(J,I)
 2100       CONTINUE
            A(I) = A(I)/NCASES
 2200    CONTINUE
         DO 2500 I = 1,ID
            DO 2400 J = I,ID
               Y(I,J) = ZERO
               DO 2300 K = 1,NCASES
                  Y(I,J) = Y(I,J)+(DATA(K,I)-A(I))*(DATA(K,J)-A(J))
 2300          CONTINUE
               Y(J,I) = Y(I,J)
 2400       CONTINUE
 2500    CONTINUE
         CALL INVERT(ID,Y,KP,KR,KC,ND)
         DO 3400 I = 1,NCASES
            DO 3300 J = 1,NCASES
               DEL(I,J) = ZERO
               DO 3200 K = 1,ID
                  DO 3100 L = 1,ID
                     DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))*Y(K,L)*
     1                                   (DATA(I,L)-DATA(J,L))
 3100             CONTINUE
 3200          CONTINUE
               DEL(I,J) = DEL(I,J)**(V/2)
               IF(DEL(I,J).GT.SUP) DEL(I,J) = SUP
               IS = IDNINT(DATA(I,ID+1))
               JS = IDNINT(DATA(J,ID+1))
               IF(IS.LE.NGRPS.AND.IS.EQ.JS) XI(IS) = XI(IS)+DEL(I,J)
 3300       CONTINUE
 3400    CONTINUE
 3405    DO 3500 I = 1,NGRPS
            XI(I) = XI(I)/(ITHGRP(I)*(ITHGRP(I)-1))
            DZ = DZ+C(I)*XI(I)
 3500    CONTINUE
         IF(DZ.LT.DL) ML = ML+1
         IF(DZ.GT.DU) MU = MU+1
 3600 CONTINUE
      PL = ONE*ML/MS
      PU = ONE*MU/MS
      RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,ID,U,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),BIN(51),DATA(1800,51)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS DATA(1,*),...,DATA(M,*) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE CONTAINED IN DATA(1,*),...,DATA(N,*)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 20 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         DO 10 K = 1,ID
            BIN(K) = DATA(J,K)
            DATA(J,K) = DATA(I,K)
            DATA(I,K) = BIN(K)
   10    CONTINUE
   20 CONTINUE
      DO 30 I = 1,N
         DATA(I,ID+1) = 0.0D0
   30 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE INVERT(N,W,KP,KR,KC,ND)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  KP(N),KR(N),KC(N),W(ND,N)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IR = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IR.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IR,L)
               W(IR,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KR(I) = IR
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KR(J).NE.KC(J)) THEN
            K = KR(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
