      PROGRAM DRIVER
C
C   THIS DRIVER PROGRAM INPUTS THE NECESSARY PARAMETERS AND THE DATA
C   TO DRIVE SUBROUTINE AGREE.  IN ADDITION, PROGRAM DRIVER PRINTS
C   OUT THE RESULTS OF THE ANALYSIS AND STORES THE RESULTS ON A FILE.
C   THE PARAMETERS ARE: 
C
C        (1) THE NAME OF THE INPUT FILE CONTAINING THE RAW DATA
C        (2) THE NAME OF THE OUTPUT FILE TO CONTAIN THE RESULTS
C        (3) THE DESCRIPTIVE HEADING FOR THE AGREEMENT ANALYSIS
C        (4) THE NUMBER OF OBSERVERS OR JUDGES (B)
C        (5) THE NUMBER OF OBJECTS OBSERVED BY EACH JUDGE (N)
C        (6) THE NUMBER OF DIMENSIONS/RESPONSES FOR EACH OBJECT (C)
C
C   THE DATA MUST BE ARRANGED AS FOLLOWS, WHERE X IS A DATA POINT, B IS
C   THE NUMBER OF JUDGES/OBSERVERS, N IS THE NUMBER OF OBJECTS OBSERVED,
C   C IS THE NUMBER OF DIMENSIONS/RESPONSES FOR EACH OF THE N OBJECTS,
C   AND THE DATA ARE IN FREE-FIELD FORMAT (I.E., THE COLUMNS OF DATA ARE
C   SEPARATED BY A COMMA OR BY ONE OR MORE SPACES): 
C
C        X(1,1,1), X(1,1,2), X(1,1,3), ..., X(1,1,C)
C        X(1,2,1), X(1,2,2), X(1,2,3), ..., X(1,2,C)
C          . . .     . . .     . . .          . . .
C        X(1,B,1), X(1,B,2), X(1,B,3), ..., X(1,B,C)
C        X(2,1,1), X(2,1,2), X(2,1,3), ..., X(2,1,C)
C        X(2,2,1), X(2,2,2), X(2,2,3), ..., X(2,2,C)
C          . . .     . . .     . . .          . . .
C        X(2,B,1), X(2,B,2), X(2,B,3), ..., X(2,B,C)
C          . . .     . . .     . . .          . . .
C        X(N,1,1), X(N,1,2), X(N,1,3), ..., X(N,1,C)
C        X(N,2,1), X(N,2,2), X(N,2,3), ..., X(N,2,C)
C          . . .     . . .     . . .          . . .
C        X(N,B,1), X(N,B,2), X(N,B,3), ..., X(N,B,C)
C
C   EXAMPLE 1: 
C      CONSIDER A PROBLEM WHERE EACH OF B = 3 JUDGES ESTIMATES THE HEIGHT,
C   WIDTH, AND DEPTH (I.E., C = 3) OF N = 5 OBJECTS, IN CENTIMETERS.  THE
C   DATA ARE GIVEN BELOW, WHERE THE JUDGE 1 ESTIMATES THE HEIGHT, WIDTH,
C   AND DEPTH OF OBJECT 1 AS 8.0, 9.2, AND 6.0, RESPECTIVELY.  JUDGE 2
C   ESTIMATES THE HEIGHT, WIDTH, AND DEPTH OF OBJECT 1 AS 8.2, 9.0, AND
C   6.5, RESPECTIVELY.  JUDGE 3 ESTIMATES THE HEIGHT, WIDTH, AND DEPTH OF
C   OBJECT 1 AS 8.2, 9.0, AND 6.5, RESPECTIVELY.  AND JUDGE 1 ESTIMATES
C   THE HEIGHT, WIDTH, AND DEPTH OF OBJECT 2 AS 10.5, 2.5, AND 11.0,
C   RESPECTIVELY.  AND SO ON . . .
C
C        [HEIGHT] [WIDTH] [DEPTH]
C
C           8.0      9.2      6.0         [OBJECT 1; JUDGE 1]
C           8.2      9.0      6.5         [OBJECT 1; JUDGE 2]
C           8.2      9.0      6.5         [OBJECT 1; JUDGE 3]
C          10.5      2.5     11.0         [OBJECT 2; JUDGE 1]
C          11.2      3.0     11.5         [OBJECT 2; JUDGE 2]
C           9.5      2.8     12.5         [OBJECT 2; JUDGE 3]
C          17.6      4.5     13.0         [OBJECT 3; JUDGE 1]
C          20.0      4.5     15.0         [OBJECT 3; JUDGE 2]
C          21.4      4.5     17.0         [OBJECT 3; JUDGE 3]
C           9.0     12.0     14.2         [OBJECT 4; JUDGE 1]
C           9.0     12.5     14.0         [OBJECT 4; JUDGE 2]
C           9.5     13.5     14.4         [OBJECT 4; JUDGE 3]
C          14.6      6.0      7.5         [OBJECT 5; JUDGE 1]
C          14.2      6.0      8.0         [OBJECT 5; JUDGE 2]
C          14.5      5.5      9.2         [OBJECT 5; JUDGE 3]
C
C   RESULTS OF THE AGREEMENT ANALYSIS: 
C   --------------------------------------------------------------------
C   HEADING: OUTPUT OF RESULTS FOR EXAMPLE 1
C
C   NUMBER OF OBSERVERS/JUDGES           : 3
C   NUMBER OF OBJECTS OBSERVED           : 5
C   NUMBER OF DIMENSIONS PER OBSERVATION : 3
C   COMPUTED VALUE OF DELTA :  .16070E+01
C   EXPECTED VALUE OF DELTA :  .82575E+01
C   VARIANCE OF DELTA       :  .11660E+01
C   SKEWNESS OF DELTA       : -.77717E+00
C   AGREEMENT MEASURE (R)   :  .80539E+00
C   PROBABILITY VALUE (P)   :  .25275E-04
C   --------------------------------------------------------------------
C
C   EXAMPLE 2: 
C      CONSIDER A SECOND PROBLEM, TAKEN FROM BERRY AND MIELKE, EDUCA-
C   TIONAL AND PSYCHOLOGICAL MEASUREMENT 48 (1988): 921-933.  IN THIS
C   CASE, B = 2 JUDGES CLASSIFY N = 5 OBJECTS INTO C = 3 CATEGORIES: 
C   'A', 'B', AND 'C'.  THE VALUE 0.7071 = SQRT(0.50) IS USED TO ENSURE
C   THAT AGREEMENT WILL BE REPRESENTED BY A ZERO AND DISAGREEMENT, BY A
C   ONE.  TO ILLUSTRATE, JUDGE 1 CLASSIFIED OBJECT 1 AS TYPE 'A', WHILE
C   JUDGE 2 CLASSIFIED OBJECT 1 AS TYPE 'B'.  JUDGE 1 CLASSIFIED OBJECT
C   2 AS TYPE 'B', ETC.  NOTE THAT, WITH TWO JUDGES AND CATEGORICAL
C   DATA, THE RESULT, R, IS IDENTICAL TO COHEN'S KAPPA STATISTIC.
C
C            [A]      [B]      [C]
C
C           0.7071   0.0      0.0         [OBJECT 1; JUDGE 1]
C           0.0      0.7071   0.0         [OBJECT 1; JUDGE 2]
C           0.0      0.7071   0.0         [OBJECT 2; JUDGE 1]
C           0.0      0.7071   0.0         [OBJECT 2; JUDGE 2]
C           0.0      0.7071   0.0         [OBJECT 3; JUDGE 1]
C           0.0      0.7071   0.0         [OBJECT 3; JUDGE 2]
C           0.0      0.0      0.7071      [OBJECT 4; JUDGE 1]
C           0.7071   0.0      0.0         [OBJECT 4; JUDGE 2]
C           0.0      0.0      0.7071      [OBJECT 5; JUDGE 1]
C           0.0      0.0      0.7071      [OBJECT 5; JUDGE 2]
C
C   RESULTS OF THE AGREEMENT ANALYSIS: 
C   --------------------------------------------------------------------
C   HEADING: OUTPUT OF RESULTS FOR EXAMPLE 2
C
C   NUMBER OF OBSERVERS/JUDGES           : 2
C   NUMBER OF OBJECTS OBSERVED           : 5
C   NUMBER OF DIMENSIONS PER OBSERVATION : 3
C   COMPUTED VALUE OF DELTA :  .40000E+00
C   EXPECTED VALUE OF DELTA :  .63999E+00
C   VARIANCE OF DELTA       :  .46399E-01
C   SKEWNESS OF DELTA       : -.40341E+00
C   AGREEMENT MEASURE (R)   :  .37500E+00
C   PROBABILITY VALUE (P)   :  .13430E+00
C   --------------------------------------------------------------------
C
C   THE CURRENT DIMENSIONS FOR PROGRAM DRIVER ARE SET FOR N = 20,
C   B = 10, AND C = 5.  TO CHANGE THE SIZE OF THE DIMENSIONS,
C   REPLACE THE VALUES IN THE 'REAL' STATEMENT BELOW; BE SURE THAT
C   'DATA' IS DIMENSIONED AT (N*B*C) AND 'D' IS DIMENSIONED AT
C   (N*B,N*B).
C
      CHARACTER HEAD*60, FIN*10, FOUT*10
      INTEGER   B, C
      REAL      DATA(1000), D(200,200), SJ(20,10,10), SJ2(20,10,10),
     +          SJ3(20,10,10), UJ(20,10,10), WI(10,10,10),
     +          YIJ(10,10,10), ZIJK(10,10,10), SIJ(10,10), SIJ2(10,10),
     +          SIJ3(10,10), UIJ(10,10), TIJ2(10,10), TIJ3(10,10),
     +          VI(10,10)
C
   10 FORMAT (A)
      WRITE  (*, *) 'ENTER NAME OF THE INPUT FILE CONTAINING THE DATA:'
      READ   (*, 10) FIN
      WRITE  (*, *) 'ENTER THE FILE NAME FOR THE OUTPUT OF RESULTS:'
      READ   (*, 10) FOUT
      WRITE  (*, *) 'ENTER THE PROBLEM HEADING (UP TO 60 CHARACTERS):'
      READ   (*, 10) HEAD
      WRITE  (*, *) 'ENTER THE NUMBER OF OBSERVERS/JUDGES (B):'
      READ   (*, *)  B
      WRITE  (*, *) 'ENTER THE NUMBER OF OBJECTS OBSERVED (N):'
      READ   (*, *)  N
      WRITE  (*, *) 'ENTER THE NUMBER OF DIMENSIONS/RESPONSES (C):'
      READ   (*, *)  C
      OPEN   (5, FILE = FIN, STATUS = 'OLD')
      DO 30 I = 1, N
         DO 20 J = 1, B
            READ (5, *, END = 80) (DATA(N*B*(K-1)+N*(J-1)+I),K = 1, C)
   20    CONTINUE
   30 CONTINUE
      CLOSE  (5)
C
      CALL AGREE (N, B, C, DATA, D, SJ, SJ2, SJ3, SIJ, SIJ2, SIJ3,
     +            TIJ2, TIJ3, UJ, UIJ, VI, WI, YIJ, ZIJK, DELTA, EDEL,
     +            VAR, GAM, R, PROB)
C
      OPEN   (6, FILE = FOUT, STATUS = 'NEW')
      WRITE  (6, 40)
   40 FORMAT (//, T5, 'RESULTS OF THE AGREEMENT ANALYSIS', /,
     +            T5, 68('-'))
      WRITE  (6, 50) HEAD, B, N, C
   50 FORMAT (T5, 'HEADING: ', A, //,
     +        T5, 'NUMBER OF OBSERVERS/JUDGES           :', I2, /,
     +        T5, 'NUMBER OF OBJECTS OBSERVED           :', I2, /,
     +        T5, 'NUMBER OF DIMENSIONS PER OBSERVATION :', I2, /)
      WRITE  (6, 60) DELTA, EDEL, VAR, GAM, R, PROB
   60 FORMAT (T5, 'COMPUTED VALUE OF DELTA :', E12.5, /,
     +        T5, 'EXPECTED VALUE OF DELTA :', E12.5, /,
     +        T5, 'VARIANCE OF DELTA       :', E12.5, /,
     +        T5, 'SKEWNESS OF DELTA       :', E12.5, /,
     +        T5, 'AGREEMENT MEASURE (R)   :', E12.5, /,
     +        T5, 'PROBABILITY VALUE (P)   :', E12.5, /,
     +        T5, 68('-'))
      WRITE  (*, 70) FOUT
   70 FORMAT (/, T2, 'N O T E:  RESULTS ARE IN OUTPUT FILE: ', A, /)
      GO TO 100
   80 WRITE  (*, 90) FIN
   90 FORMAT (/, T2, 'E R R O R: END-OF-FILE FOUND ON FILE: ', A, /)
  100 END
C
C
      SUBROUTINE AGREE (N, B, C, DATA, D, SJ, SJ2, SJ3, SIJ, SIJ2,
     +   SIJ3, TIJ2, TIJ3, UJ, UIJ, VI, WI, YIJ, ZIJK, DELTA, EDEL,
     +   VAR, GAM, R, PROB)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST: 
C
C        N      - INPUT, NUMBER OF OBJECTS OBSERVED
C        B      - INPUT, NUMBER OF OBSERVERS/JUDGES
C        C      - INPUT, NUMBER OF DIMENSIONS/RESPONSES
C        DATA   - INPUT, MATRIX CONTAINING THE RAW SCORE VALUES
C
C        D      - WORKING ARRAY, CONTAINING THE DIFFERENCE SCORES
C        SJ     - WORKING ARRAY, CONTAINING SJ INTERMEDIATE VALUES
C        SJ2    - WORKING ARRAY, CONTAINING SJ2 INTERMEDIATE VALUES
C        SJ3    - WORKING ARRAY, CONTAINING SJ3 INTERMEDIATE VALUES
C        SIJ    - WORKING ARRAY, CONTAINING SIJ INTERMEDIATE VALUES
C        SIJ2   - WORKING ARRAY, CONTAINING SIJ2 INTERMEDIATE VALUES
C        SIJ3   - WORKING ARRAY, CONTAINING SIJ3 INTERMEDIATE VALUES
C        TIJ2   - WORKING ARRAY, CONTAINING TIJ2 INTERMEDIATE VALUES
C        TIJ3   - WORKING ARRAY, CONTAINING TIJ3 INTERMEDIATE VALUES
C        UJ     - WORKING ARRAY, CONTAINING UJ INTERMEDIATE VALUES
C        UIJ    - WORKING ARRAY, CONTAINING UIJ INTERMEDIATE VALUES
C        VI     - WORKING ARRAY, CONTAINING VI INTERMEDIATE VALUES
C        WI     - WORKING ARRAY, CONTAINING WI INTERMEDIATE VALUES
C        YIJ    - WORKING ARRAY, CONTAINING YIJ INTERMEDIATE VALUES
C        ZIJK   - WORKING ARRAY, CONTAINING ZIJK INTERMEDIATE VALUES
C
C        DELTA  - OUTPUT, OBSERVED (REALIZED) VALUE OF DELTA
C        EDEL   - OUTPUT, EXPECTED (MEAN) VALUE OF DELTA
C        VAR    - OUTPUT, VARIANCE OF THE DELTA DISTRIBUTION
C        GAM    - OUTPUT, SKEWNESS OF THE DELTA DISTRIBUTION
C        R      - OUTPUT, DELTA-BASED AGREEMENT COEFFICIENT
C        PROB   - OUTPUT, PROBABILITY OF AGREEMENT COEFFICIENT
C
C   USAGE: 
C
C        SUBROUTINE AGREE CALCULATES THE VALUE FOR DELTA AND THE
C        VALUE FOR THE COEFFICIENT OF AGREEMENT, R.  IN ADDITION,
C        EXACT VALUES FOR THE MEAN (EDEL), VARIANCE (VAR), AND
C        SKEWNESS (GAM) OF THE DELTA DISTRIBUTION ARE COMPUTED.
C
      INTEGER B, C
      REAL    DATA(N,B,C), D(N*B,N*B), SJ(N,B,B), SJ2(N,B,B), VI(B,B),
     +        SJ3(N,B,B), UJ(N,B,B), WI(B,B,B), YIJ(B,B,B), UIJ(B,B),
     +        ZIJK(B,B,B), SIJ(B,B), SIJ2(B,B), SIJ3(B,B), TIJ2(B,B),
     +        TIJ3(B,B)
C
      ZERO = 0.0
      DO 50 I = 1, N
         DO 40 J = 1, B
            DO 30 K = I, N
               LO = 1
               IF (I.EQ.K) LO = J
               DO 20 L = LO, B
                  IJ = B*(I-1)+J
                  KL = B*(K-1)+L
                  D(IJ,KL) = ZERO
                  DO 10 M = 1, C
                     D(IJ,KL) = D(IJ,KL)+(DATA(I,J,M)-DATA(K,L,M))**2
   10             CONTINUE
                  D(IJ,KL) = D(IJ,KL)**0.5
                  D(KL,IJ) = D(IJ,KL)
   20          CONTINUE
   30       CONTINUE
   40    CONTINUE
   50 CONTINUE
      DO 90 IS = 1, B
         DO 80 IR = 1, B
            IF (IR.EQ.IS) GO TO 80
            SIJ(IR,IS)  = ZERO
            SIJ2(IR,IS) = ZERO
            SIJ3(IR,IS) = ZERO
            DO 70 I = 1, N
               SJ(I,IR,IS)  = ZERO
               SJ2(I,IR,IS) = ZERO
               SJ3(I,IR,IS) = ZERO
               DO 60 J = 1, N
                  IRR = (I-1)*B+IR
                  JSS = (J-1)*B+IS
                  SJ(I,IR,IS)  = SJ(I,IR,IS)+D(IRR,JSS)
                  SJ2(I,IR,IS) = SJ2(I,IR,IS)+D(IRR,JSS)**2
                  SJ3(I,IR,IS) = SJ3(I,IR,IS)+D(IRR,JSS)**3
   60          CONTINUE
               SIJ(IR,IS)  = SIJ(IR,IS)+SJ(I,IR,IS)
               SIJ2(IR,IS) = SIJ2(IR,IS)+SJ2(I,IR,IS)
               SIJ3(IR,IS) = SIJ3(IR,IS)+SJ3(I,IR,IS)
   70       CONTINUE
   80    CONTINUE
   90 CONTINUE
      T2 = ZERO
      IF (B.LE.2) GO TO 160
         DO 150 IT = 3, B
            DO 140 IS = 2, IT-1
               DO 130 IR = 1, IS-1
                  WI(IR,IS,IT)   = ZERO
                  WI(IS,IR,IT)   = ZERO
                  WI(IT,IR,IS)   = ZERO
                  YIJ(IR,IS,IT)  = ZERO
                  ZIJK(IR,IS,IT) = ZERO
                  DO 120 I = 1, N
                     WI(IR,IS,IT) = WI(IR,IS,IT)+SJ(I,IR,IS)*SJ(I,IR,IT)
                     WI(IS,IR,IT) = WI(IS,IR,IT)+SJ(I,IS,IR)*SJ(I,IS,IT)
                     WI(IT,IR,IS) = WI(IT,IR,IS)+SJ(I,IT,IR)*SJ(I,IT,IS)
                     DO 110 J = 1, N
                        IRR = B*(I-1)+IR
                        JSS = B*(J-1)+IS
                        JTT = B*(J-1)+IT
                        ISS = B*(I-1)+IS
                        YIJ(IR,IS,IT) = YIJ(IR,IS,IT)+D(IRR,JSS)*
     +                                  SJ(I,IR,IT)*SJ(J,IS,IT)+
     +                                  D(IRR,JTT)*SJ(I,IR,IS)*
     +                                  SJ(J,IT,IS)+D(ISS,JTT)*
     +                                  SJ(I,IS,IR)*SJ(J,IT,IR)
                        DO 100 K = 1, N
                           KTT = (K-1)*B+IT
                           ZIJK(IR,IS,IT) = ZIJK(IR,IS,IT)+D(IRR,JSS)*
     +                                      D(IRR,KTT)*D(JSS,KTT)
  100                   CONTINUE
  110                CONTINUE
  120             CONTINUE
                  T2 = T2+SIJ(IR,IS)*SIJ(IR,IT)*SIJ(IS,IT)-SIJ(IS,IT)*
     +                 WI(IR,IS,IT)*N-SIJ(IR,IT)*WI(IS,IR,IT)*N-
     +                 SIJ(IR,IS)*WI(IT,IR,IS)*N+YIJ(IR,IS,IT)*N*N-
     +                 ZIJK(IR,IS,IT)*N*N*N
  130          CONTINUE
  140       CONTINUE
  150    CONTINUE
         T2 = 6.0*T2/(N-1)
  160 DO 200 IS = 2, B
         DO 190 IR = 1, IS-1
            TIJ2(IR,IS) = ZERO
            TIJ3(IR,IS) = ZERO
            VI(IR,IS)   = ZERO
            UIJ(IR,IS)  = ZERO
            DO 180 I = 1, N
               TIJ2(IR,IS) = TIJ2(IR,IS)+SJ(I,IR,IS)**2+SJ(I,IS,IR)**2
               TIJ3(IR,IS) = TIJ3(IR,IS)+SJ(I,IR,IS)**3+SJ(I,IS,IR)**3
               VI(IR,IS)   = VI(IR,IS)+SJ(I,IR,IS)*SJ2(I,IR,IS)+
     +                       SJ(I,IS,IR)*SJ2(I,IS,IR)
               UJ(I,IR,IS) = ZERO
               DO 170 J = 1, N
                  IRR = B*(I-1)+IR
                  JSS = B*(J-1)+IS
                  UJ(I,IR,IS) = UJ(I,IR,IS)+D(IRR,JSS)*SJ(I,IR,IS)*
     +                          SJ(J,IS,IR)
  170          CONTINUE
               UIJ(IR,IS) = UIJ(IR,IS)+UJ(I,IR,IS)
  180       CONTINUE
  190    CONTINUE
  200 CONTINUE
      EDEL = ZERO
      VAR  = ZERO
      T1   = ZERO
      DO 220 IS = 2, B
         DO 210 IR = 1, IS-1
            IF (N.GT.2) THEN
               T1 = T1+4.0*SIJ(IR,IS)**3-SIJ(IR,IS)*TIJ2(IR,IS)*6.0*
     +              N+UIJ(IR,IS)*6.0*N*N+TIJ3(IR,IS)*2.0*N*N+
     +              SIJ(IR,IS)*SIJ2(IR,IS)*3.0*N*N-VI(IR,IS)*3.0*
     +              N*N*N+SIJ3(IR,IS)*N**4
            END IF
            EDEL = EDEL+SIJ(IR,IS)
            VAR  = VAR+SIJ(IR,IS)*SIJ(IR,IS)-TIJ2(IR,IS)*
     +             N+SIJ2(IR,IS)*N*N
  210    CONTINUE
  220 CONTINUE
      IF (N.GT.2) T1 = T1/(N-2)
      FAC = N*B*(B-1)/2.0
      CON = 1.0/(FAC*N)
      VAR = VAR*CON*CON/(N-1)
      GAM = (CON**3*(T1-T2)/(N-1))/SQRT(VAR**3)
      EDEL = CON*EDEL
      DELTA = ZERO
      DO 250 IS = 2, B
         DO 240 IR = 1, IS-1
            DO 230 I = 1, N
               IRR = (I-1)*B+IR
               ISS = (I-1)*B+IS
               DELTA = DELTA+D(IRR,ISS)
  230       CONTINUE
  240    CONTINUE
  250 CONTINUE
      DELTA = DELTA/(FAC)
      T = (DELTA-EDEL)/SQRT(VAR)
      R = 1.0-(DELTA/EDEL)
C
      CALL PVALUE (T, GAM, PROB)
C
      RETURN
      END
C
C
      SUBROUTINE PVALUE (T, GAM, PROB)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST: 
C
C        T    - INPUT, STANDARDIZED TEST STATISTIC
C        GAM  - INPUT, SKEWNESS OF THE DELTA DISTRIBUTION
C        PROB - OUTPUT, PROBABILITY OF THE TEST STATISTIC
C
C   USAGE: 
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A
C        VALUE OF T BEING LESS THAN OR EQUAL TO THE OBSERVED
C        VALUE OF T.
C
      ZERO = 0.0
      PI = 4.0*ATAN(1.0)
      IF (ABS(GAM).LT.0.01) GO TO 50
      R = 2.0/ABS(GAM)
      D = R*R
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      F = R*R+10.0
      U = (2.0*F-1.0)*LOG(F)/2.0-F+LOG(2.0*PI)/2.0-LOG(D)+1.0/
     +    (12.0*F)-1.0/(360.0*F*F*F)
      G1 = 0.045
      G2 = 0.090
      G3 = 0.015
      H1 = ZERO
      H2 = ZERO
      A  = R*R-1.0
      B  = R*R*(LOG(R)-1.0)-U
      W  = -1.99/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF (T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+9.0
      DO 20 I = 1,99
         H1 = H1+EXP(A*LOG(R+X+G1*(2.0*I-1.0))-R*(X+G1*(2.0*I-1.0))+B)
         H2 = H2+EXP(A*LOG(R+X+G2*I)-R*(X+G2*I)+B)
   20 CONTINUE
      H0  = EXP(A*LOG(R+X)-R*X+B)
      H3  = EXP(A*LOG(R+Y)-R*Y+B)
      H4  = EXP(A*LOG(R+X+G1*199.0)-R*(X+G1*199.0)+B)
      PROB = 1.0-G3*(H0+(H1+H4)*4.0+2.0*H2+H3)
      RETURN
   30 IF (T.GT.W) THEN
         PROB = 1.0
         RETURN
      END IF
      X = T-9.0
      Y = T
      DO 40 I = 1,99
         H1 = H1+EXP(A*LOG(R-X-G1*(2.0*I-1.0))+R*(X+G1*(2.0*I-1.0))+B)
         H2 = H2+EXP(A*LOG(R-X-G2*I)+R*(X+G2*I)+B)
   40 CONTINUE
      H0 = EXP(A*LOG(R-X)+R*X+B)
      H3 = EXP(A*LOG(R-Y)+R*Y+B)
      H4 = EXP(A*LOG(R-X-G1*199.0)+R*(X+G1*199.0)+B)
      PROB = G3*(H0+(H1+H4)*4.0+2.0*H2+H3)
      RETURN
   50 E1 =  0.319381530
      E2 = -0.356563782
      E3 =  1.781477937
      E4 = -1.821255978
      E5 =  1.330274429
      H  =  0.231641900
      W  =  1.0/(H*ABS(T)+1.0)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*EXP(-T*T/2.0)/SQRT(PI*2.0)
      IF (T.GT.ZERO) PROB = 1.0-PROB
      RETURN
      END
