      PROGRAM EMRBP11
      IMPLICIT REAL*8(A-H,O-Z)
C
C      THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND THE EXACT P-VALUE
C      FOR AN ANALYSIS OF A RANDOMIZED BLOCK EXPERIMENT WITH 11 BLOCKS (B = 11).
C      THE PRESENT MAXIMUM VALUES OF G AND R ARE 10 AND 15 [AS A CONSEQUENCE
C      OF (3!)(3!)(3!)(3!)(3!)(3!)(3!)(3!)(3!)(3!) = 60,466,176, G = 3 IS
C      LIKELY AT A MAXIMUM; HOWEVER, R MAY BE INCREASED IF REQUIRED].
C
C    THIS PROGRAM IS CAPABLE OF PERFORMING (1) REPEATED EMRBP11 ANALYSES
C    IN ONE OPERATION, (2) ALIGNMENT WITHIN BLOCKS, (3) DISTANCE FUNCTION
C    COMMENSURATION, AND (4) C(G,H) RANKS TEST.
C
C         PROGRAM MODIFIED 2/8/2003
C
      DIMENSION  AD(15),XM(11,15),DATA(10,11,15),X(10,11,15),S(10,11,11)
      CHARACTER HEAD*80,FIN*40,FOUT*40
C
C       THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE WITH EACH
C       OBJECT'S R RESPONSE VALUES ON A SEPARATE LINE AS FOLLOWS:
C
C         A(1,1,1),A(1,1,2),...,A(1,1,R)
C         A(1,2,1),A(1,2,2),...,A(1,2,R)
C                  ...
C         A(1,B,1),A(1,B,2),...,A(1,B,R)
C         A(2,1,1),A(2,1,2),...,A(2,1,R)
C                  ...
C         A(2,B,1),A(2,B,2),...,A(2,B,R)
C                  ...
C         A(G,1,1),A(G,1,2),...,A(G,1,R)
C                  ...
C         A(G,B,1),A(G,B,2),...,A(G,B,R)
C
C   THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C   G IS # OF GROUPS, B IS # OF BLOCKS, R IS # OF RESPONSES, IA = 1 IMPLIES
C   ALIGNMENT, IC = 1 IMPLIES COMMENSURATION, AND LR = 1 IMPLIES C(G,H)
C   RANKS TEST.   NOTE: ASSOCIATE G, B AND R WITH KG, KB AND KR IN PROGRAM.
C
      ZERO=0.0D0
      ONE=1.0D0
      TWO=2.0D0
      PREC=1.0D-12
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,'(A)') FIN
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,'(A)') FOUT
      OPEN(5,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    2 READ(5,'(A)',END=5) HEAD
      GO TO 10
    5 GO TO 900
   10 READ(5,*) V,KG,KB,KR,IA,IC,LR,H
      DO 20 I=1,KG
         DO 15 J=1,KB
            READ(5,*,END=870) (DATA(I,J,K),K=1,KR)
   15    CONTINUE
   20 CONTINUE
      IF(KB.NE.11) STOP 'NUMBER OF BLOCKS NOT EQUAL TO 11'
      WRITE(6,21) HEAD,V,KR,KB,KG
   21 FORMAT(' ANALYSIS OF RANDOMIZED BLOCK EXPERIMENT (EMRBP11): '/
     1     4X,A/4X,'DISTANCE FUNCTION EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES, ',I3,' BLOCKS, ',I3,' GROUPS.')
      IF(LR.EQ.1) THEN
         IA = 0
         IC = 0
         CALL RANK(KG,KB,KR,H,DATA)
         GO TO 60
      END IF
      IF(IA.EQ.0) GO TO 40
      DO 25 I=1,KG
         DO 24 J=1,KB
            DO 23 K=1,KR
               X(I,J,K)=DATA(I,J,K)
   23       CONTINUE
   24    CONTINUE
   25 CONTINUE
      DO 30 J=1,KB
         DO 29 K=1,KR
            A1=1.0D200
            SUM1=A1
            DO 28 I1=1,KG
               SUM=ZERO
               DO 27 I2=1,KG
                  SUM=SUM+DABS(DATA(I2,J,K)-X(I1,J,K))
   27          CONTINUE
               IF(SUM.LT.A1) THEN
                  DM1=X(I1,J,K)
                  A1=SUM
                  A2=A1*1.0000000001D0
               END IF
               IF(SUM.LT.A2.AND.DM1.NE.X(I1,J,K)) THEN
                  DM2=X(I1,J,K)
                  SUM1=SUM
               END IF
   28       CONTINUE
            IF(SUM1.GT.A2) DM2=DM1
            XM(J,K)=(DM1+DM2)/2
   29    CONTINUE
   30 CONTINUE
      DO 35 I=1,KG
         DO 34 J=1,KB
            DO 33 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)-XM(J,K)
   33       CONTINUE
   34    CONTINUE
   35 CONTINUE
   40 IF(IC.EQ.0.OR.KR.EQ.1) GO TO 60
      DO 45 K=1,KR
         AD(K)=ZERO
         DO 44 I1=1,KG
            DO 43 I2=1,KG
               DO 42 J1=2,KB
                  DO 41 J2=1,J1-1
                     AD(K)=AD(K)+(DABS(DATA(I1,J1,K)-DATA(I2,J2,K)))**V
   41             CONTINUE
   42          CONTINUE
   43       CONTINUE
   44    CONTINUE
         AD(K)=AD(K)**(ONE/V)
   45 CONTINUE
      DO 50 I=1,KG
         DO 49 J=1,KB
            DO 48 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)/AD(K)
   48       CONTINUE
   49    CONTINUE
   50 CONTINUE
   60 DELTA1=ZERO
      DO 200 I=1,KG
         DO 175 J=1,KB-1
            DO 150 K=J+1,KB
               S(I,J,K)=ZERO
               DO 125 L=1,KR
                  S(I,J,K)=S(I,J,K)+(DATA(I,J,L)-DATA(I,K,L))**2
  125          CONTINUE
               DELTA1=DELTA1+S(I,J,K)**(V/TWO)
  150       CONTINUE
  175    CONTINUE
  200 CONTINUE
      C=TWO/(KG*KB*(KB-1))
      DELTA1=DELTA1*(ONE+PREC)*C
      CALL FACT2(KG,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,DATA)
      PROB = ONE*KOUNTB/KOUNTA
      WRITE(6,570) LR
  570 FORMAT(T5,'1 (0) IMPLIES (NO) C(G,H) RANKS TEST:',I3)
      IF(LR.EQ.1) THEN
        WRITE(6,575) H
  575   FORMAT(T5,'C(G,H) RANKS TEST EXPONENT H =',G15.8)
      ELSE
        WRITE(6,580)
  580   FORMAT(T5,' ')
      ENDIF
      WRITE(6,585) IA
  585 FORMAT(T5,'1 (0) INDICATES (NO) MEDIAN ALIGNMENT:',I3)
      WRITE(6,590) IC
  590 FORMAT(T5,'1 (0) IMPLIES (NO) AVE. D.F. COMMENSURATION:',I3)
      WRITE(6,595) DELTA1
  595 FORMAT(T5,'OBSERVED MRBP STATISTIC =',G15.8)
      WRITE(6,600) KOUNTA
  600 FORMAT(T5,'TOTAL NUMBER OF PERMUTATIONS =',I9)
      WRITE(6,650) PROB
  650 FORMAT(T5,'EXACT MRBP P-VALUE =',G15.8)
      WRITE(6,700)
  700 FORMAT('********************************************************')
      GO TO 2
  870 WRITE(*,880)
  880 FORMAT(' E R R O R:  END-OF-FILE ENCOUNTERED.')
      WRITE(*,890) KG,KB,KR
  890 FORMAT(' CHECK THESE:  G = ',I3,' B = ',I3,' R = ',I3)
      STOP
  900 CLOSE(5)
      CLOSE(6)
      END
C
C
      SUBROUTINE RANK(KG,KB,KR,H,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  RKS(10),DATA(10,11,15)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      YM = ONE*(KG+1)/2
      DO 80 J = 1,KB
         DO 70 K = 1,KR
            CL = 1.0D30
            DO 3 I = 1,KG
               IF(DATA(I,J,K).LT.CL) CL = DATA(I,J,K)
    3       CONTINUE
            DO 7 I = 1,KG
               DATA(I,J,K) = DATA(I,J,K)-CL+ONE
    7       CONTINUE
            PHI = ONE+1.0D-12
            A1 = ZERO
            A2 = ZERO
            A3 = ONE*KG-0.1D0
            B1 = ZERO-1.0D30
            B2 = 1.0D30
   10       IF(A2.GT.A3)  GO TO 50
            DO 20 I = 1,KG
               IF(DATA(I,J,K).GT.B1.AND.DATA(I,J,K).LT.B2)
     1           B2 = DATA(I,J,K)*PHI
   20       CONTINUE
            DO 30 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  A1 = A1+1
   30       CONTINUE
            A4 = A2+(A1+1)/2
            DO 40 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  RKS(I) = A4
   40       CONTINUE
            A2 = A2+A1
            A1 = ZERO
            B1 = B2
            B2 = 1.0D30
            GO TO 10
   50       DO 60 I = 1,KG
               W = DABS(RKS(I)-YM)
               IF(W.LT.0.001D0)  THEN
                  DATA(I,J,K) = ZERO
               ELSE
                  DATA(I,J,K) = (RKS(I)-YM)*DABS(RKS(I)-YM)**(H-1)
               END IF
   60       CONTINUE
   70    CONTINUE
   80 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE FACT2(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT2 = 0
      KOUNTA = 0
      KOUNTB = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT2.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P2(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT3(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,DATA)
      KOUNT2 = KOUNT2+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT3(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT3 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT3.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P3(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT4(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,DATA)
      KOUNT3 = KOUNT3+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT4(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT4 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT4.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P4(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT5(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,P4,DATA)
      KOUNT4 = KOUNT4+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT5(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,P4,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT5 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT5.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P5(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT6(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,P4,P5,DATA)
      KOUNT5 = KOUNT5+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT6(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT6 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT6.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P6(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT7(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,P2,P3,P4,P5,P6,DATA)
      KOUNT6 = KOUNT6+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT7(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,P6,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT7 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT7.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P7(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT8(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +           P2,P3,P4,P5,P6,P7,DATA)
      KOUNT7 = KOUNT7+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT8(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,P6,P7,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),P8(10),
     +         A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT8 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT8.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P8(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT9(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +           P2,P3,P4,P5,P6,P7,P8,DATA)
      KOUNT8 = KOUNT8+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT9(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,P6,P7,P8,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),
     +         P8(10),P9(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT9 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT9.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P9(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT10(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +           P2,P3,P4,P5,P6,P7,P8,P9,DATA)
      KOUNT9 = KOUNT9+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT10(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,P6,P7,P8,P9,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),
     +         P8(10),P9(10),P10(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT10 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT10.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P10(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL FACT11(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +           P2,P3,P4,P5,P6,P7,P8,P9,P10,DATA)
      KOUNT10 = KOUNT10+1
      GO TO 60
      END
C
C
      SUBROUTINE FACT11(N,KB,KR,KOUNTA,KOUNTB,DELTA1,C,V,
     +                 P2,P3,P4,P5,P6,P7,P8,P9,P10,DATA)
C
      DOUBLE PRECISION  DELTA1,C,V,DATA(10,11,15)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),
     +         P8(10),P9(10),P10(10),P11(10),A(10,10)
C
      K = N
      L = N
      DO 50 I = 1,K-1
         A(I,1) = I
         DO 40 J = 2,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-1
   50 CONTINUE
      KOUNT11 = 0
   60 I = K-1
      NMARKS = 2
      IF(KOUNT11.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+1
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P11(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL DVAL(N,KB,KR,KOUNTB,DELTA1,C,V,P2,P3,P4,P5,P6,P7,P8,P9,P10,
     +          P11,DATA)
      KOUNT11 = KOUNT11+1
      KOUNTA = KOUNTA+1
      GO TO 60
      END
C
C
      SUBROUTINE DVAL(KG,KB,KR,KOUNTB,DELTA1,C,V,
     +                P2,P3,P4,P5,P6,P7,P8,P9,P10,P11,DATA)
C
      DOUBLE PRECISION  ZERO,ONE,DELTA,DELTA1,C,V,DATA(10,11,15),
     +                  X(10,11,15),S(10,11,11)
      INTEGER  P2(10),P3(10),P4(10),P5(10),P6(10),P7(10),P8(10),P9(10),
     +         P10(10),P11(10)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DO 100 I = 1,KG
         DO 50 K = 1,KR
            X(I,1,K) = DATA(I,1,K)
   50    CONTINUE
  100 CONTINUE
      DO 200 I = 1,KG
         DO 150 K = 1,KR
            X(I,2,K) = DATA(P2(I),2,K)
  150    CONTINUE
  200 CONTINUE
      DO 300 I = 1,KG
         DO 250 K = 1,KR
            X(I,3,K) = DATA(P3(I),3,K)
  250    CONTINUE
  300 CONTINUE
      DO 400 I = 1,KG
         DO 350 K = 1,KR
            X(I,4,K) = DATA(P4(I),4,K)
  350    CONTINUE
  400 CONTINUE
      DO 500 I = 1,KG
         DO 450 K = 1,KR
            X(I,5,K) = DATA(P5(I),5,K)
  450    CONTINUE
  500 CONTINUE
      DO 600 I = 1,KG
         DO 550 K = 1,KR
            X(I,6,K) = DATA(P6(I),6,K)
  550    CONTINUE
  600 CONTINUE
      DO 700 I = 1,KG
         DO 650 K = 1,KR
            X(I,7,K) = DATA(P7(I),7,K)
  650    CONTINUE
  700 CONTINUE
      DO 800 I = 1,KG
         DO 750 K = 1,KR
            X(I,8,K) = DATA(P8(I),8,K)
  750    CONTINUE
  800 CONTINUE
      DO 900 I = 1,KG
         DO 850 K = 1,KR
            X(I,9,K) = DATA(P9(I),9,K)
  850    CONTINUE
  900 CONTINUE
      DO 1000 I = 1,KG
         DO 950 K = 1,KR
            X(I,10,K) = DATA(P10(I),10,K)
  950    CONTINUE
 1000 CONTINUE
      DO 1100 I = 1,KG
         DO 1050 K = 1,KR
            X(I,11,K) = DATA(P11(I),11,K)
 1050    CONTINUE
 1100 CONTINUE
      DELTA=ZERO
      DO 1200 I=1,KG
         DO 1175 J=1,KB-1
            DO 1150 K=J+1,KB
               S(I,J,K)=ZERO
               DO 1125 L=1,KR
                  S(I,J,K)=S(I,J,K)+(X(I,J,L)-X(I,K,L))**2
 1125          CONTINUE
               DELTA=DELTA+S(I,J,K)**(V/2)
 1150       CONTINUE
 1175    CONTINUE
 1200 CONTINUE
      DELTA = DELTA*C
      IF(DELTA.LT.DELTA1) KOUNTB = KOUNTB+1
      RETURN
      END
