      PROGRAM EOSMP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         PERMUTATION TEST FOR MATCHED PAIRS (EOSMP) YIELDS EXACT
C         FISHER-PITMAN TYPE P-VALUES AND CLASSICAL STUDENT t-TEST
C         P-VALUES (POWER OF RANKS TEST IS INCLUDED AS A SPECIAL CASE).
C         AN OPTION OF THIS PROGRAM IS THE ONE-SAMPLE TEST.
C         THIS PROGRAM IS CAPABLE OF MULTIPLE RUNS.
C         PROGRAM MODIFIED 4/25/02
C
      DIMENSION  D(300,2),X(100),Y(100),R(100)
      CHARACTER  FIN*40,FOUT*40,HEAD*80
      INTEGER  OLDN
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PREC = 1.0D-12
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,1) FIN
    1 FORMAT(A)
      WRITE(*,*) 'ENTER: FILE NAME FOR OUTPUT RESULTS'
      READ(*,1) FOUT
      OPEN(1,FILE=FIN)
      OPEN(2,FILE=FOUT,STATUS='NEW')
    5 READ(1,1,END=7) HEAD
      GO TO 10
    7 GO TO 200
   10 READ(1,*) OLDN,KM,ZM,LR,H
      IF(KM.EQ.2) GO TO 13
      READ(1,*) (D(I,1),I=1,OLDN)
      DO 12 I = 1,OLDN
         D(I,1) = (D(I,1)-ZM)/2
         D(I,2) = ZERO-D(I,1)
   12 CONTINUE
      GO TO 14
   13 READ(1,*) ((D(I,J),J = 1,2),I = 1,OLDN)
   14 N = 0
      DO 15 I = 1,OLDN
         DIFF = D(I,1)-D(I,2)
         IF(DIFF.NE.ZERO) THEN
            N = N+1
            X(N) = DIFF
         END IF
   15 CONTINUE
      IF(N.LT.3) THEN
         WRITE(*,*) 'N LESS THAN THREE'
         GO TO 300
      END IF
      CT1 = ZERO
      CT2 = ZERO
      DO 16 I = 1,N
         CT1 = CT1+X(I)
         CT2 = CT2+X(I)**2
   16 CONTINUE
      CT = CT1/DSQRT((CT2*N-CT1**2)/(N-1))
      IF(LR.NE.1) GO TO 30
      DO 17 I = 1,N
         Y(I) = DABS(X(I))
   17 CONTINUE
      A1 = ZERO
      A2 = ZERO
      A3 = ONE*N-0.1D0
      B1 = ZERO
      B2 = 1.0D30
   18 IF(A2.GT.A3) GO TO 22
      DO 19 I = 1,N
         IF(Y(I).GT.B1.AND.Y(I).LT.B2) B2 = Y(I)*(ONE+PREC)
   19 CONTINUE
      DO 20 I = 1,N
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) A1 = A1+1
   20 CONTINUE
      A4 = A2+(A1+1)/2
      DO 21 I = 1,N
         W = DABS(ONE-Y(I)/B2)
         IF(W.LT.1.0D-11) R(I) = A4
   21 CONTINUE
      A2 = A2+A1
      A1 = ZERO
      B1 = B2
      B2 = 1.0D30
      GO TO 18
   22 DO 23 I = 1,N
         X(I) = (X(I)/Y(I))*R(I)**H
   23 CONTINUE
   30 YM = 10D30
      DO 32 I = 1,N
         Y(I) = DABS(X(I))
         IF(Y(I).LT.YM) YM = Y(I)
   32 CONTINUE
      TO = ZERO
      VT = ZERO
      DO 35 I = 1,N
         TO = TO+X(I)
         VT = VT+X(I)**2
   35 CONTINUE
      TA = TO/DSQRT(VT)
      WW = DABS(TA)
      CALL NORMAL(WW,PZ)
      IF(TA.GT.ZERO) THEN
         PZ1 = PZ
         PZ2 = ONE-PZ
      ELSE
         PZ1 = ONE-PZ
         PZ2 = PZ
      END IF
      PZ3 = DMIN1(PZ1*2,PZ2*2)
      CALL COMB(N,KOUNT1,YM,TO,P1,P2,Y)
      P3 = DMIN1(P1*2,P2*2,ONE)
   93 WRITE(2,1) HEAD
      WRITE(2,95) 'RESULTS OF EOSMP ANALYSIS'
   95 FORMAT(T24,A,/)
      IF(KM.EQ.1) THEN
         WRITE(2,97) 'EOSMP IS A ONE-SAMPLE TEST'
         WRITE(2,110) 'THE HYPOTHESIZED CENTER VALUE =', ZM
      ELSE
         WRITE(2,97) 'EOSMP IS A MATCHED-PAIRS TEST'
      END IF
   97 FORMAT(T15,A)
      WRITE(2,1) ' '
      WRITE(2,100) 'NUMBER OF PERMUTATIONS =',KOUNT1
  100 FORMAT(T10,A,I9)
      WRITE(2,1) ' OBSERVED STATISTIC IS THE SUM OF ALL VALUES'
      WRITE(2,110) 'OBSERVED STATISTIC =', TO
      IF(LR.NE.1) GO TO 105
      WRITE(2,110) 'EXPONENT FOR POWER OF RANKS TEST =', H
  105 WRITE(2,97) 'THE FOLLOWING P-VALUES ARE EXACT:'
      WRITE(2,110) 'P-VALUE OF A > OR = OBSERVED STATISTIC =', P1
      WRITE(2,110) 'P-VALUE OF A < OR = OBSERVED STATISTIC =', P2
      WRITE(2,110) 'TWO-SIDED P-VALUE =', P3
      WRITE(2,110) 'OBSERVED ASYMPTOTIC STATISTIC =', TA
      WRITE(2,97) 'THE FOLLOWING ASYMPTOTIC P-VALUES ARE APPROXIMATE:'
      WRITE(2,110) 'P-VALUE OF A > OR = OBSERVED STATISTIC =', PZ1
      WRITE(2,110) 'P-VALUE OF A < OR = OBSERVED STATISTIC =', PZ2
      WRITE(2,110) 'TWO-SIDED P-VALUE =', PZ3
  110 FORMAT(T10,A,G20.8)
      WRITE(2,1) ' '
      WRITE(2,120) 'NUMBER OF VALUES OR PAIRS =', OLDN
      WRITE(2,120) 'NUMBER OF NON-ZERO VALUES OR DIFFERENCES =', N
  120 FORMAT(T10,A,I4)
      DF = ONE*(N-1)
      CTS = CT
      CALL TPV(CTS,DF,PCT)
      PCTU = PCT/2
      IF(CT.GT.ZERO) THEN
         PCT1 = PCTU
         PCT2 = ONE-PCTU
      ELSE
         PCT1 = ONE-PCTU
         PCT2 = PCTU
      END IF
      WRITE(2,110) 'OBSERVED STUDENT t-TEST STATISTIC =', CT
      WRITE(2,97) 'THE FOLLOWING ARE STUDENT t-TEST P-VALUES:'
      WRITE(2,110) 'P-VALUE OF A > OR = STUDENT STATISTIC =', PCT1
      WRITE(2,110) 'P-VALUE OF A < OR = STUDENT STATISTIC =', PCT2
      WRITE(2,110) 'STUDENT t-TEST TWO-SIDED P-VALUE =', PCT
      WRITE(2,130)
  130 FORMAT('********************************************************')
      GO TO 5
  200 CLOSE(1)
      CLOSE(2)
  300 END
C
C
      SUBROUTINE COMB(N,KOUNT1,YM,TO,P2,P3,X)
C
      DOUBLE PRECISION  YM,TO,T2,T3,P2,P3,ONE,X(100)
      INTEGER  M(30)
C
      ONE = 1.0D0
      T2 = TO-YM*1.0D-5
      T3 = TO+YM*1.0D-5
      DO 10 I = 1,N
         M(I) = 1
   10 CONTINUE
      KOUNT1 = 0
      KOUNT2 = 0
      KOUNT3 = 0
      KOUNT4 = 0
      NMINUS = 0
      LAST = 0
      GO TO 50
   20 J = 0
   30 J = J+1
      IF(MOD(KOUNT4,2).EQ.1) GO TO 40
      KOUNT4 = KOUNT4/2
      GO TO 30
   40 LAST = M(J)
      M(J) = -LAST
   50 KOUNT1 = KOUNT1+1
      KOUNT4 = KOUNT1
      NMINUS = NMINUS+LAST
      CALL XACT(N,KOUNT2,KOUNT3,T2,T3,X,M)
      IF(NMINUS.EQ.1.AND.M(N).EQ.-1) GO TO 60
      GO TO 20
   60 P2 = ONE*KOUNT2/KOUNT1
      P3 = ONE*KOUNT3/KOUNT1
      RETURN
      END
C
C
      SUBROUTINE XACT(N,KOUNT2,KOUNT3,T2,T3,X,M)
C
      DOUBLE PRECISION  ZERO,T2,T3,SUM,X(100)
      INTEGER  M(30)
C
      ZERO = 0.0D0
      SUM = ZERO
      DO 10 I = 1,N
            SUM = SUM+X(I)*M(I)
   10 CONTINUE
      IF(SUM.GT.T2) KOUNT2 = KOUNT2+1
      IF(SUM.LT.T3) KOUNT3 = KOUNT3+1
      RETURN
      END


C
C
      SUBROUTINE TPV(X,DD,P)
      DOUBLE PRECISION ZERO,ONE,X,Y,DN,DD,A,B,P,BETAIN
C
      ZERO = 0.0D0
      ONE = 1.0D0
      X = X*X
      DN = ONE
      IF(DD.LE.ZERO) THEN
         STOP 'DF IS NOT POSITIVE'
      END IF
      IF(X.GT.ZERO) THEN
         GO TO 20
      ELSE
         P = ONE
         GO TO 30
      END IF
   20 A = DN/2
      B = DD/2
      Y = A*X/(A*X+B)
      P = ONE-BETAIN(Y,A,B)
   30 RETURN
      END
C
      DOUBLE PRECISION FUNCTION BETAIN(X,A,B)
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL INDEX
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A+B
      CX = ONE-X
      IF(A.GE.AB*X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB+CX*AB)
      RX = XX/CX
   30 TEMP = BB-AI
      IF(NS.EQ.0) RX = XX
   40 TERM = TERM*TEMP*RX/(AA+AI)
      BETAIN = BETAIN+TERM
      TEMP = DABS(TERM)
      IF(TEMP.LE.TOL.AND.TEMP.LE.TOL*BETAIN) GO TO 50
      AI = AI+1
      NS = NS-1
      IF(NS.GE.0) GO TO 30
      TEMP = AB
      AB = AB+1
      GO TO 40
   50 BETAIN = BETAIN*DEXP(AA*DLOG(XX)+(BB-1)*DLOG(CX)-DLNBTA(AA,BB))/AA
      IF(INDEX) BETAIN = ONE-BETAIN
      END
C
      DOUBLE PRECISION FUNCTION DLNBTA(A,B)
      DOUBLE PRECISION  A,B,DLNGAM
      DLNBTA = DLNGAM(A)+DLNGAM(B)-DLNGAM(A+B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
      ONE = 1.0D0
      Z = X-1
      D = ONE
      PI = DATAN(ONE)*4
      F = Z+10
      DO 10 J = 1,9
         D = D*(Z+J)
   10 CONTINUE
      DLNGAM = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +         -ONE/((F**3)*360)+ONE/((F**5)*1260)-ONE/((F**7)*1680)
     +         +ONE/((F**9)*1188)
      END
C
C
      SUBROUTINE NORMAL(Z,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,G,A,B,C,E,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-738.0D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-738.0D0) E = DEXP(G)
      RETURN
      END
