      PROGRAM GMRPT
C
C         THE GENERALIZED MULTI-RESPONSE PERMUTATION TECHNIQUE (GMRPT)
C         TEST STATISTIC IS CALCULATED IN THIS FORTRAN-77 PROGRAM.  THIS
C         TEST IS DESIGNED TO ANALYZE MULTI-RESPONSE DATA AT THE ORDINAL
C         OR HIGHER LEVEL AND IS BASED ON THE WEIGHTED AVERAGE DISTANCE
C         (OR ANY OTHER SPECIFIED NORM) BETWEEN ALL PAIRS OF POINTS IN
C         AN R-DIMENSIONAL SPACE CORRESPONDING TO R MEASURED RESPONSES
C         FROM EACH OBJECT IN A FINITE POPULATION).  SAMPLE SIZE STRUCTURE
C         AND SAMPLE-OBJECT ASSOCIATION INFORMATION IS MAINTAINED IN THIS
C         PROGRAM.  THE ONLY RESTRICTION ON GMRPT IS THAT THE C(I,J)'S
C         AND THE D(I,J)'S MUST BE SYMMETRIC.  SINCE GMRPT IS SO GENERAL,
C         MODIFICATIONS OF THIS BASIC PROGRAM ARE ANTICIPATED FOR SPECIFIC
C         PURPOSES.  THIS MATERIAL IS TAKEN FROM THE APPENDIX OF THE
C         FOLLOWING REFERENCE:  Mielke, P.W. (1978).  Clarification and
C         appropriate inferences for Mantel and Valand's nonparametric
C         multivariate analysis technique.  Biometrics, 34, 277-282.
C
C         PROGRAM MODIFIED 1/16/2002
C
      DOUBLE PRECISION  V,C3O,D3O,CLIT1,DLIT1,CLIT2,DLIT2,CLIT3,DLIT3,
     +   O1,P1,O2,P2,O3,P3,O4,P4,STAT,E1,VAR,T,GAM,PROB,RHO
      CHARACTER  HEAD*80,FIN*40,FOUT*40
C
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,1000) FIN
 1000 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1000) FOUT
      CALL DIST(N,ID,V,HEAD,FIN,C3O,D3O,CLIT1,DLIT1,CLIT2,
     +   DLIT2,CLIT3,DLIT3,O1,P1,O2,P2,O3,P3,O4,P4,STAT)
      CALL CALC(N,C3O,D3O,CLIT1,DLIT1,CLIT2,DLIT2,CLIT3,DLIT3,
     +   O1,P1,O2,P2,O3,P3,O4,P4,STAT,E1,VAR,T,GAM)
      RHO = 1.0D0-STAT/E1
      CALL PVALUE(T,GAM,PROB)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,1000) HEAD
      HEAD = 'GENERALIZED MULTI-RESPONSE PERMUTATION TECHNIQUE (GMRPT)'
      WRITE(6,1100) HEAD
 1100 FORMAT(/,T19,A,/)
      WRITE(6,1200) 'INPUT HAS ', N,' OBSERVATIONS'
 1200 FORMAT(T10,A,I3,A)
      WRITE(6,1200) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1800) 'STANDARDIZED TEST STATISTIC =', T
 1800 FORMAT(T10,A,G20.8)
      WRITE(6,1800) 'DISTANCE EXPONENT =', V
      WRITE(6,1800) 'OBSERVED STATISTIC =', STAT
      WRITE(6,1800) 'EXPECTED STATISTIC =', E1
      WRITE(6,1800) 'VARIANCE OF STATISTIC =', VAR
      WRITE(6,1800) 'SKEWNESS OF STATISTIC =', GAM
      WRITE(6,1800) 'THE AGREEMENT MEASURE =', RHO
      WRITE(6,1900) 'P-VALUE OF A SMALLER OR EQUAL STATISTIC =', PROB
 1900 FORMAT(/,T10,A,G20.8)
      END
C
      SUBROUTINE DIST(N,ID,V,HEAD,FIN,C3O,D3O,CLIT1,DLIT1,
     +   CLIT2,DLIT2,CLIT3,DLIT3,O1,P1,O2,P2,O3,P3,O4,P4,STAT)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES . . . .
C
      DOUBLE PRECISION  V,ZERO,ONE,CLIT1,DLIT1,CLIT2,DLIT2,CLIT3,DLIT3,
     +   O1,P1,O2,P2,O3,P3,O4,P4,DATA(50,16),C(50,50),D(50,50),
     +   CSUB1(50),DSUB1(50),CSUB2(50),DSUB2(50),C3O,D3O,STAT
      CHARACTER  HEAD*80,FIN*40
C
      ZERO = 0.0D0
      ONE = 1.0D0
      OPEN(8,FILE=FIN)
      READ(8,'(A)') HEAD
      READ(8,*) N,ID,V
      IF(N.LT.6) STOP 'TOTAL NUMBER OF CASES < 6'
      DO 600 I = 1,N
         READ(8,*,END=700) (DATA(I,J),J = 1,ID)
  600 CONTINUE
      CLOSE(8)
      GO TO 900
  700 WRITE(*,800) 'E R R O R:  END-OF-FILE ENCOUNTERED.'
  800 FORMAT(/,T3,A)
      STOP
  900 CLIT1 = ZERO
      DLIT1 = ZERO
      CLIT2 = ZERO
      DLIT2 = ZERO
      CLIT3 = ZERO
      DLIT3 = ZERO
      STAT = ZERO
      DO 1000 I = 1,N
         CSUB1(I) = ZERO
         DSUB1(I) = ZERO
         CSUB2(I) = ZERO
         DSUB2(I) = ZERO
 1000 CONTINUE
      DO 1150 I = 1,N
         DO 1100 J = 1,N
            C(I,J) = ZERO
            D(I,J) = ZERO
            DO 1050 K = 1,ID
               D(I,J) = D(I,J)+(DATA(I,K)-DATA(J,K))**2
 1050       CONTINUE
            D(I,J) = D(I,J)**(V/2)
 1100    CONTINUE
 1150 CONTINUE
C
C         SPECIFICATION OF C(I,J)'S . . . .
C       (THE PRESENT SIMPLE EXAMPLE YIELDS MRSP)
C
      DO 1200 I = 1,N-1
         C(I,I+1) = ONE
         C(I+1,I) = ONE
 1200 CONTINUE
      DO 1500 I = 1,N
         DO 1400 J = 1,N
            CLIT1 = CLIT1+C(I,J)
            DLIT1 = DLIT1+D(I,J)
            CLIT2 = CLIT2+C(I,J)**2
            DLIT2 = DLIT2+D(I,J)**2
            CLIT3 = CLIT3+C(I,J)**3
            DLIT3 = DLIT3+D(I,J)**3
            CSUB1(I) = CSUB1(I)+C(I,J)
            DSUB1(I) = DSUB1(I)+D(I,J)
            CSUB2(I) = CSUB2(I)+C(I,J)**2
            DSUB2(I) = DSUB2(I)+D(I,J)**2
            STAT = STAT+C(I,J)*D(I,J)
 1400    CONTINUE
 1500 CONTINUE
      C3O = ZERO
      D3O = ZERO
      DO 1720 I = 1,N-2
         DO 1710 J = I+1,N-1
            DO 1700 K = J+1,N
               C3O = C3O+C(I,J)*C(I,K)*C(J,K)
               D3O = D3O+D(I,J)*D(I,K)*D(J,K)
 1700       CONTINUE
 1710    CONTINUE
 1720 CONTINUE
      C3O = C3O*6/(ONE*N*(N-1)*(N-2))
      D3O = D3O*6/(ONE*N*(N-1)*(N-2))
      O1 = ZERO
      P1 = ZERO
      O2 = ZERO
      P2 = ZERO
      O3 = ZERO
      P3 = ZERO
      O4 = ZERO
      P4 = ZERO
      DO 1800 I = 1,N
         O1 = O1+CSUB1(I)**2
         P1 = P1+DSUB1(I)**2
         O2 = O2+CSUB1(I)*CSUB2(I)
         P2 = P2+DSUB1(I)*DSUB2(I)
         O4 = O4+CSUB1(I)**3
         P4 = P4+DSUB1(I)**3
 1800 CONTINUE
      DO 1910 I = 1,N-1
         DO 1900 J = I+1,N
            O3 = O3+CSUB1(I)*CSUB1(J)*C(I,J)
            P3 = P3+DSUB1(I)*DSUB1(J)*D(I,J)
 1900    CONTINUE
 1910 CONTINUE
      RETURN
      END
C
      SUBROUTINE CALC(N,C3O,D3O,CLIT1,DLIT1,CLIT2,DLIT2,CLIT3,DLIT3,
     +   O1,P1,O2,P2,O3,P3,O4,P4,STAT,E1,VAR,T,GAM)
C
C         SUBROUTINE CALC CALCULATES ALL THE REMAINING PARAMETERS
C         TOGETHER WITH THE FIRST THREE MOMENTS OF THE SAMPLING
C         DISTRIBUTION . . . .
C
      DOUBLE PRECISION  C3O,D3O,CLIT1,DLIT1,CLIT2,DLIT2,CLIT3,DLIT3,
     +   O1,P1,O2,P2,O3,P3,O4,P4,STAT,E1,VAR,T,GAM,COF1,COF2,COF3,COF4,
     +   COF5,C1,D1,C2,D2,C3,D3,C2S,D2S,C3S,D3S,C2SS,D2SS,C3SS,D3SS,
     +   C3SSS,D3SSS,C3SSSS,D3SSSS,C3OO,D3OO,C3OOO,D3OOO,E2,E3,SQV
C
      COF1 = 1.0D0*N*(N-1)
      C1 = CLIT1/COF1
      D1 = DLIT1/COF1
      C2 = CLIT2/COF1
      D2 = DLIT2/COF1
      C3 = CLIT3/COF1
      D3 = DLIT3/COF1
      COF2 = COF1*(N-2)
      C2S = (O1-CLIT2)/COF2
      D2S = (P1-DLIT2)/COF2
      C3S = (O2-CLIT3)/COF2
      D3S = (P2-DLIT3)/COF2
      COF3 = COF2*(N-3)
      C2SS = (CLIT1**2-C2S*COF2*4-CLIT2*2)/COF3
      D2SS = (DLIT1**2-D2S*COF2*4-DLIT2*2)/COF3
      C3SS = (CLIT1*CLIT2-C3S*COF2*4-CLIT3*2)/COF3
      D3SS = (DLIT1*DLIT2-D3S*COF2*4-DLIT3*2)/COF3
      C3OO = (O3*2-C3S*COF2*2-C3O*COF2-CLIT3)/COF3
      D3OO = (P3*2-D3S*COF2*2-D3O*COF2-DLIT3)/COF3
      C3OOO = (O4-C3S*COF2*3-CLIT3)/COF3
      D3OOO = (P4-D3S*COF2*3-DLIT3)/COF3
      COF4 = COF3*(N-4)
      C3SSS = (CLIT1*C2S*COF2-C3OO*COF3*4-C3OOO*COF3*2-
     +        C3S*COF2*4-C3O*COF2*2)/COF4
      D3SSS = (DLIT1*D2S*COF2-D3OO*COF3*4-D3OOO*COF3*2-
     +        D3S*COF2*4-D3O*COF2*2)/COF4
      COF5 = COF4*(N-5)
      C3SSSS = (CLIT1*C2SS*COF3-C3SSS*COF4*8-C3SS*COF3*4-
     +         C3OO*COF3*8)/COF5
      D3SSSS = (DLIT1*D2SS*COF3-D3SSS*COF4*8-D3SS*COF3*4-
     +         D3OO*COF3*8)/COF5
      E1 = C1*D1*COF1
      E2 = C2*D2*COF1*2+C2S*D2S*COF2*4+C2SS*D2SS*COF3
      E3 = C3*D3*COF1*4+C3S*D3S*COF2*24+C3SS*D3SS*COF3*6+C3O*D3O*COF2*8+
     +     C3OO*D3OO*COF3*24+C3OOO*D3OOO*COF3*8+C3SSS*D3SSS*COF4*12+
     +     C3SSSS*D3SSSS*COF5
      VAR = E2-E1**2
      SQV = DSQRT(VAR)
      GAM = (E3-VAR*E1*3-E1**3)/SQV**3
      T = (STAT-E1)/SQV
      RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION  T,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H1N,H2,H3
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
