      PROGRAM MRPC
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THIS ALGORITHM OBTAINS THE APPROXIMATE P-VALUE OF MULTI-RESPONSE
C         PERMUTATION COMPARISON (MRPC) OF SIMILARITY BETWEEN TWO SAMPLES
C         BASED ON THE AN APPROXIMATING NORMAL DISTRIBUTION BASED ON THE
C         EXACT MEAN AND VARIANCE TO ANALYZE MULTIVARIATE DATA AT ORDINAL
C         (OR HIGHER) LEVELS AND DEPENDS ON AVERAGE STANDARDIZED EUCLIDEAN
C         DISTANCE BETWEEN OBJECT POSITIONS OF THE TWO SAMPLES HAVING
C         SIZES N1 AND N2, RESPECTIVELY, WITHIN G A PRIORI DISJOINT FINITE
C         GROUPS OF n1(I) > 0 AND n2(I) > 0 OBJECTS, RESPECTIVELY, FOR
C         I = 1,...,G WHERE EACH POSITION IS A POINT IN AN R-DIMENSIONAL
C         SPACE (THUS THE ARGUMENTS OF EACH R-DIMENSIONAL POINT CORRESPOND
C         TO THE R MEASURED RESPONSES OF EACH OBJECT IN THE FINITE POPULATION
C         OF OBJECTS).  THE PRESENT MAXIMUM VALUES OF G, N1, N2, AND R
C         IN THIS PROGRAM ARE 60, 1200, 1200, AND 15, RESPECTIVELY.  THIS
C         PROGRAM IS CAPABLE OF MULTIPLE RUNS.
C
C         PROGRAM MODIFIED 2/24/2008
C
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = sqrt(n1(I)n2(I))/sum(sqrt(n1(I)n2(I)))
C            CFORM = 2:  C(I) = n1(I)n2(I)/sum(n1(I)n2(I))
C            CFORM = 3:  C(I) = (n1(I)+N2(I))/(N1+N2)
C            CFORM = 4:  C(I) = 1/sum(1)
C       CFORM IS PASSED AS THE FOURTH PARAMETER ON THE SECOND RECORD
C       IN THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED AND
C       STORED IN ARRAY C(I).
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION L1(0:60),L2(0:60),M1(60),M2(60),J1(1200),J2(1200),
     +          C(60),ZO(60),X1(1200,15),X2(1200,15),D(1200,1200)
      INTEGER   CFORM
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
  110 READ(3,100,END=120) HEAD
      GO TO 130
  120 GO TO 2000
  130 READ(3,*) N1,N2,ID,CFORM,NGRPS
      READ(3,*) (M1(I),I = 1,NGRPS)
      READ(3,*) (M2(I),I = 1,NGRPS)
      DO 140 I = 1,N1
         READ(3,*) (X1(I,J),J = 1,ID)
  140 CONTINUE
      DO 150 I = 1,N2
         READ(3,*) (X2(I,J),J = 1,ID)
  150 CONTINUE
      GO TO (155,170,185,200) CFORM
  155 FK = ZERO
      DO 160 I = 1,NGRPS
         FK = FK+DSQRT(ONE*M1(I)*M2(I))
  160 CONTINUE
      DO 165 I = 1,NGRPS
         C(I) = DSQRT(ONE*M1(I)*M2(I))/FK
  165 CONTINUE
      GO TO 220
  170 FK = ZERO
      DO 175 I = 1,NGRPS
         FK = FK+ONE*M1(I)*M2(I)
  175 CONTINUE
      DO 180 I = 1,NGRPS
         C(I) = ONE*M1(I)*M2(I)/FK
  180 CONTINUE
      GO TO 220
  185 FK = ZERO
      DO 190 I = 1,NGRPS
         FK = FK+ONE*M1(I)+M2(I)
  190 CONTINUE
      DO 195 I = 1,NGRPS
         C(I) = (ONE*M1(I)+M2(I))/FK
  195 CONTINUE
      GO TO 220
  200 DO 205 I = 1,NGRPS
         C(I) = ONE/NGRPS
  205 CONTINUE
  220 L1(0) = 0
      L2(0) = 0
      DO 225 I = 1,NGRPS
         L1(I) = L1(I-1)+M1(I)
         L2(I) = L2(I-1)+M2(I)
  225 CONTINUE
      DO 230 I = 1,N1
         J1(I) = I
  230 CONTINUE
      DO 235 I = 1,N2
         J2(I) = I
  235 CONTINUE
      A = ZERO
      BIJ = ZERO
      DO 250 I = 1,N1
         DO 245 J = 1,N2
            D(I,J) = ZERO
            DO 240 K = 1,ID
               D(I,J) = D(I,J)+(X1(I,K)-X2(J,K))**2
  240       CONTINUE
            D(I,J) = DSQRT(D(I,J))
            A = A+D(I,J)
            BIJ = BIJ+D(I,J)**2
  245    CONTINUE
  250 CONTINUE
      EW = A/(N1*N2)
      WO = ZERO
      C2 = ZERO
      DO 280 K = 1,NGRPS
         ZO(K) = ZERO
         DO 270 I = L1(K-1)+1,L1(K)
            DO 260 J = L2(K-1)+1,L2(K)
               ZO(K) = ZO(K)+D(I,J)
  260       CONTINUE
  270    CONTINUE
         ZO(K) = ZO(K)/(M1(K)*M2(K))
         WO = WO+C(K)*ZO(K)
         C2 = C2+C(K)**2
  280 CONTINUE
      RO = ONE-WO/EW
      BI = ZERO
      BJ = ZERO
      DO 350 II = 1,N1
         DO 345 JJ1 = 1,N2
            DO 340 JJ2 = 1,N2
               BI = BI+D(II,JJ1)*D(II,JJ2)
  340       CONTINUE
  345    CONTINUE
  350 CONTINUE
      BI = BI-BIJ
      DO 370 JJ = 1,N2
         DO 365 II1 = 1,N1
            DO 360 II2 = 1,N1
               BJ = BJ+D(II1,JJ)*D(II2,JJ)
  360       CONTINUE
  365    CONTINUE
  370 CONTINUE
      BJ = BJ-BIJ
      B = A*A-BI-BJ-BIJ
      BIJ = BIJ/N1/N2
      BI = BI/N1/N2/(N2-1)
      BJ = BJ/N1/(N1-1)/N2
      B = B/N1/(N1-1)/N2/(N2-1)
      EW2 = ZERO
      DO 380 K = 1,NGRPS
         EW2 = EW2+C(K)**2*(BIJ+BJ*(M1(K)-1)+BI*(M2(K)-1)+
     +             B*(M1(K)-1)*(M2(K)-1))/M1(K)/M2(K)
  380 CONTINUE
      EW2 = EW2+B*(ONE-C2)
      EVAR = EW2-EW**2
      T = (WO-EW)/DSQRT(EVAR)
      CALL PNORMAL(T,PN)
C
      WRITE(6,100) HEAD
      HEAD = 'TWO-SAMPLE MULTI-RESPONSE PERMUTATION COMPARISON'
      WRITE(6,1100) HEAD
 1100 FORMAT(T2,A)
      WRITE(6,1200) 'INPUT HAS ',N1,' AND ',N2,' CASES IN SAMPLES ',
     +              '#1 AND #2, RESPECTIVELY'
 1200 FORMAT(T2,A,I4,A,I4,A,A)
      WRITE(6,1250) 'THERE WERE ',ID,' DIMENSION(S)'
 1250 FORMAT(T2,A,I3,A)
      WRITE(6,1300) 'IN THE ', NGRPS,' GROUPS OF SAMPLE #1 AND'
 1300 FORMAT(T2,A,I2,A)
      WRITE(6,1300) 'THE ', NGRPS,' CORRESPONDING GROUPS OF SAMPLE #2'
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = sqrt(n1(I)n2(I))/sum(sqrt(n1(J)n2(J)))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = n1(I)n2(I)/sum(n1(J)n2(J))'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = (n1(I)+n2(I))/(N1+N2)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = 1/sum(1)'
 1450 FORMAT(T10,A)
 1500 WRITE(6,100) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZES ', M1(I),' AND ',
     +                 M2(I),' HAS AN AVERAGE DISTANCE =', ZO(I)
 1600    FORMAT(T2,A,I2,A,I4,A,I4,A,G16.8)
 1700 CONTINUE
      WRITE(6,100) ' '
 1800 FORMAT(T10,A,G16.8E3)
      WRITE(6,1800) 'APPROXIMATE NORMAL P-VALUE = ',PN
      WRITE(6,1800) 'OBSERVED VALUE OF W = ', WO
      WRITE(6,1800) 'EXACT MEAN OF W = ', EW
      WRITE(6,1800) 'EXACT VARIANCE OF W = ', EVAR
      WRITE(6,1800) 'AGREEMENT MEASURE ASSOCIATED WITH WO = ',RO
      WRITE(6,1800) 'STANDARDIZED STATISTIC = ', T
      WRITE(6,1850)
 1850 FORMAT('********************************************************')
      GO TO 110
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE PNORMAL(Z,PROB)
C
C         SUBROUTINE PNORMAL CALCULATES THE PROBABILITY THAT A N(0,1)
C        VALUE OF  Z IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,A,B,C,G,H,U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      PI = DATAN(ONE)*4
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = DEXP(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
