      PROGRAM NKS
C
C  PROGRAM NKS YIELDS THE POINT PROBABILITY AND EXACT ONE-SIDED AND TWO-SIDED
C  P-VALUES OF THE OBSERVED SUM OF RANKS (S) FOR A GIVEN ONE OF K OBJECTS WHERE
C  EACH OF N JUDGES PROVIDES DISTINCT RANKS (1,...,K) TO EACH OF THE K OBJECTS
C  (FOR EXAMPLE THE K OBJECTS ARE RANKED FROM WORST TO BEST).  NOTE: S = JO.
C
C       Berry, K.J., and P.W. Mielke (2001).  Probabilities of the sum of
C          N ranks assigned to one of K objects by N independent judges.
C          PERCEPTUAL AND MOTOR SKILLS, 93, 154-156.
C
      DOUBLE PRECISION  ZERO,ONE,C,LCOMB,P1,P2,PS,PX
      WRITE(*,*) 'N IS # OF JUDGES, K IS # OF RANKED OBJECTS, AND S IS'
      WRITE(*,*) '    A SPECIFIED OBJECT''S OBSERVED SUM OF RANKS.'
      WRITE(*,*) 'THE RANGE OF S IS BOUNDED BY N BELOW AND NK ABOVE.'
      WRITE(*,*) 'ENTER N, K & S:'
      READ(*,*) N,K,JO
      IF(JO.LT.N) STOP 'S IS LESS THAN N.'
      IF(JO.GT.N*K) STOP 'S IS GREATER THAN NK.'
      ZERO = 0.0D0
      ONE = 1.0D0
      IO = JO-N
      C = DLOG(ONE*K)*N
      K = K-1
      LO = MIN0(IO,N*K-IO)
      P1 = ZERO
      DO 100 L = 0,LO
         MX = MIN0(N,L/(K+1))
         PS = ZERO
         DO 90 M = 0,MX
            PX = (-ONE)**M*DEXP(LCOMB(N,M)+LCOMB(L-M*(K+1)+N-1,N-1)-C)
            PS = PS+PX
   90    CONTINUE
         P1 = P1+PS
  100 CONTINUE
      P2 = DMIN1(P1*2,ONE)
      WRITE(*,*) 'POINT PROBABILITY = ',PS
      WRITE(*,*) 'ONE-SIDED P-VALUE = ',P1
      WRITE(*,*) 'TWO-SIDED P-VALUE = ',P2
      END
C
C
      DOUBLE PRECISION FUNCTION LCOMB(N,M)
C
C        FUNCTION LCOMB YIELDS THE NATURAL LOGARITHM (BASE e) OF THE
C                COMBINATION OF N OBJECTS TAKEN M AT A TIME
C
      DOUBLE PRECISION  ONE,PI,R,Z,D,F,U1,U2,U3
C
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      R = DLOG(PI*2)/2
      Z = ONE+N
      D = Z
      F = Z+10
      DO 10 I = 1,9
         D = D*(Z+I)
   10 CONTINUE
      U1 = (F*2-1)*DLOG(F)/2-F+R-DLOG(D)+ONE/(F*12)-ONE/(F**3*360)
     +        +ONE/(F**5*1260)-ONE/(F**7*1680)+ONE/(F**9*1188)
      Z = ONE+M
      D = Z
      F = Z+10
      DO 20 I = 1,9
         D = D*(Z+I)
   20 CONTINUE
      U2 = (F*2-1)*DLOG(F)/2-F+R-DLOG(D)+ONE/(F*12)-ONE/(F**3*360)
     +        +ONE/(F**5*1260)-ONE/(F**7*1680)+ONE/(F**9*1188)
      Z = ONE+N-M
      D = Z
      F = Z+10
      DO 30 I = 1,9
         D = D*(Z+I)
   30 CONTINUE
      U3 = (F*2-1)*DLOG(F)/2-F+R-DLOG(D)+ONE/(F*12)-ONE/(F**3*360)
     +        +ONE/(F**5*1260)-ONE/(F**7*1680)+ONE/(F**9*1180)
      LCOMB = U1-U2-U3
      RETURN
      END
