      PROGRAM RGMRPT
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE GENERALIZED MULTI-RESPONSE PERMUTATION TECHNIQUE (RGMRPT)
C         TEST STATISTIC IS CALCULATED IN THIS FORTRAN-77 PROGRAM.  THIS
C         TEST IS DESIGNED TO ANALYZE MULTI-RESPONSE DATA AT THE ORDINAL
C         OR HIGHER LEVEL AND IS BASED ON THE WEIGHTED AVERAGE DISTANCE
C         (OR ANY OTHER SPECIFIED NORM) BETWEEN ALL PAIRS OF N POINTS IN
C         AN R-DIMENSIONAL SPACE CORRESPONDING TO R MEASURED RESPONSES
C         FROM EACH OBJECT IN A FINITE POPULATION).  SAMPLE SIZE STRUCTURE
C         AND SAMPLE-OBJECT ASSOCIATION INFORMATION IS MAINTAINED IN THIS
C         PROGRAM.  THE ONLY RESTRICTION ON GMRPT IS THAT THE C(I,J)'S
C         AND THE D(I,J)'S MUST BE SYMMETRIC.  SINCE GMRPT IS SO GENERAL,
C         MODIFICATIONS OF THIS BASIC PROGRAM ARE ANTICIPATED FOR SPECIFIC
C         PURPOSES.  THE PRESENT MAXIMUM VALUES OF R AND N IN THIS PROGRAM
C         ARE 40 AND 120, RESPECTIVELY.  THIS MATERIAL IS TAKEN FROM THE
C         APPENDIX OF THE FOLLOWING REFERENCE:  Mielke, P.W. (1978).
C         Clarification and appropriate inferences for Mantel and Valand's
C         nonparametric multivariate analysis technique.  Biometrics,
C         34, 277-282.
C
C         PROGRAM MODIFIED 1/16/2002
C
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  U(17),DATA(120,40),DEL(120,120),TD(120,120),
     +           CDEL(120,120)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,100) FIN
  100 FORMAT(A)
      OPEN(3,FILE=FIN)
      READ(3,'(A)') HEAD
      READ(3,*) N,ID,V
      DO 600 I = 1,N
         READ(3,*,END=700) (DATA(I,J),J = 1,ID)
  600 CONTINUE
      CLOSE(3)
      GO TO 900
  700 WRITE(*,800) 'E R R O R:  END-OF-FILE ENCOUNTERED.'
  800 FORMAT(/,T3,A)
      STOP
  900 WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      KK = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 1104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 1102 JJ = 1,KK
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
 1102    CONTINUE
         U(II) = S
 1104 CONTINUE
      DO 1130 I = 1,N
         DO 1120 J = 1,N
            CDEL(I,J) = ZERO
            DEL(I,J) = ZERO
            DO 1110 K = 1,ID
               DEL(I,J) = DEL(I,J)+(DATA(I,K)-DATA(J,K))**2
 1110       CONTINUE
            DEL(I,J) = DEL(I,J)**(V/2)
 1120    CONTINUE
 1130 CONTINUE
C
C         SPECIFICATION OF CDEL(I,J)'S . . . .
C       (THE PRESENT SIMPLE EXAMPLE YIELDS MRSP)
C
      DO 1140 I = 1,N-1
         CDEL(I,I+1) = ONE
         CDEL(I+1,I) = ONE
 1140 CONTINUE
      DBAR = ZERO
      DO 1160 I = 1,N
         DO 1150 J = 1,N
            DBAR = DBAR+CDEL(I,J)*DEL(I,J)
 1150    CONTINUE
 1160 CONTINUE
      DBAR1 = DBAR*(ONE+1.0D-12)
      DBAR2 = DBAR*(ONE-1.0D-12)
      MP1 = 0
      MP2 = 0
      DO 1190 IS = 1,MS
         CALL SAMPWOR(N,N,U,DEL,TD)
         DX = ZERO
         DO 1180 I = 1,N
            DO 1170 J = 1,N
               DX = DX+CDEL(I,J)*TD(I,J)
 1170       CONTINUE
 1180    CONTINUE
         IF(DX.LT.DBAR1) MP1 = MP1+1
         IF(DX.GT.DBAR2) MP2 = MP2+1
 1190 CONTINUE
      PROB1 = ONE*MP1/MS
      PROB2 = ONE*MP2/MS
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,100) HEAD
      HEAD = 'GENERALIZED MULTI-RESPONSE PERMUTATION TECHNIQUE (RGMRPT)'
      WRITE(6,1500) HEAD
 1500 FORMAT(/,T19,A,/)
      WRITE(6,1600) 'INPUT HAS ', N,' OBSERVATIONS'
 1600 FORMAT(T10,A,I3,A)
      WRITE(6,1600) 'THERE WERE', ID, ' DIMENSION(S)'
      WRITE(6,1700) 'RANDOM NUMBER GENERATOR SEED =',ISEED
 1700 FORMAT(T10,A,I9)
      WRITE(6,1700) 'NUMBER OF SIMULATIONS =',MS
      WRITE(6,1800) 'DISTANCE EXPONENT =', V
 1800 FORMAT(T10,A,G20.8)
      WRITE(6,1800) 'OBSERVED STATISTIC =', DBAR
      WRITE(6,1900) 'P-VALUE OF A SMALLER OR EQUAL STATISTIC =', PROB1
      WRITE(6,1900) 'P-VALUE OF A LARGER OR EQUAL STATISTIC =', PROB2
 1900 FORMAT(/,T10,A,G20.8)
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,DEL,TD)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),L(120),DEL(120,120),TD(120,120)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     L: REAL INPUT WHICH CONTAINS 1,...,M POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE CONTAINED IN L(1),...,L(N)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,M
         L(I) = I
   10 CONTINUE
      DO 20 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         LBIN = L(J)
         L(J) = L(I)
         L(I) =  LBIN
   20 CONTINUE
      DO 40 I = 1,N
         DO 30 J = 1,N
            TD(I,J) = DEL(L(I),L(J))
   30    CONTINUE
   40 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
