      PROGRAM EDCPV
      IMPLICIT REAL*8(A-H,O-Z)
C
C  PROGRAM EDCPV IMPLEMENTS THE DISCRETE DATA METHOD FOR OBTAINING AN EXACT
C  P-VALUE ANALOGOUS TO FISHER'S CONTINUOUS DATA METHOD FOR COMBINING P-VALUES
C  WHEN TWO OR MORE TESTS ASSOCIATED WITH DISCRETE P-VALUES ARE COMBINED.
C  LET N(I) BE THE NUMBER OF DISCRETE PROBABILITIES ASSOCIATED WITH THE ITH
C  OF K TESTS WHERE K = 1,...,9 (THE PROGRAM IS EASILY EXTENDED TO K > 9 WITH
C  TRIVIAL PROGRAMMING IF THIS IS EVER NEEDED).  THE EXACT P-VALUES OF THIS
C  VERSION ARE FOR FISHER'S EXACT, EXACT CHI-SQUARE AND EXACT LOG-LR TESTS.
C  IF K = 1, THEN THIS PROGRAM REDUCES TO OBTAINING A SINGLE P-VALUE.  THE
C  PROGRAM CAN BE EASILY MODIFIED TO HANDLE A WIDE VARIETY OF CASES WHICH
C  EXAMINE MULTIPLE TESTS.  THE K TESTS OF THIS PROGRAM MUST BE INDEPENDENT
C  OF ONE ANOTHER.  SIZE IS AN OBVIOUS RESTRICTION OF THIS METHOD.  THIS IS
C  SIMPLY THAT M = N(1)*...*N(K) CAN NOT BE TOO LARGE SINCE IT IS ALL TOO
C  COMMON THAT SINGLE EXACT TEST'S P-VALUE MAY BE BASED ON A VALUE N(1) THAT
C  PROHIBITS ITS USE; PRESENT RESTRICTION IS N(I) < 1501 FOR I = 1,...,K.
C
      CHARACTER FIN*40,FOUT*40
      DIMENSION N(9),XO(9),ZO(9),PO(9),X(9,1500),Z(9,1500),P(9,1500)
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'THE NUMBER OF INDEPENDENT TESTS:'
      READ(*,*) K
      IF(K.GT.9) STOP 'K IS GREATER THAN 9'
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,100) FOUT
      OPEN(1,FILE=FIN)
      OPEN(8,FILE=FOUT,STATUS='NEW')
      DO 200 I = 1,K
        READ(1,*) N(I),XO(I),ZO(I),PO(I)
        DO 150 J = 1,N(I)
         READ(1,*) X(I,J),Z(I,J),P(I,J)
  150  CONTINUE
  200 CONTINUE
      POP = ONE
      XOS = ZERO
      ZOS = ZERO
      FEP = ZERO
      EXP = ZERO
      EZP = ZERO
      ZM = ONE
      DO 300 I = 1,K
       POP = POP*PO(I)
       XOS = XOS+XO(I)
       ZOS = ZOS+ZO(I)
       ZM = ZM*N(I)
  300 CONTINUE
      WRITE(8,400) 'TOTAL NUMBER OF LOOPS = ',ZM
  400 FORMAT(T3,A,D20.13)
      POP = POP*(ONE+1.0D-11)
      XOS = XOS*(ONE-1.0D-12)
      ZOS = ZOS*(ONE-1.0D-12)
      IF(K.EQ.1) GO TO 1000
      IF(K.EQ.2) GO TO 2000
      IF(K.EQ.3) GO TO 3000
      IF(K.EQ.4) GO TO 4000
      IF(K.EQ.5) GO TO 5000
      IF(K.EQ.6) GO TO 6000
      IF(K.EQ.7) GO TO 7000
      IF(K.EQ.8) GO TO 8000
      IF(K.EQ.9) GO TO 9000
 1000 DO 1100 J1 = 1,N(1)
       AP = P(1,J1)
       AX = X(1,J1)
       AZ = Z(1,J1)
       IF(AP.LT.POP) FEP = FEP+AP
       IF(AX.GT.XOS) EXP = EXP+AP
       IF(AZ.GT.ZOS) EZP = EZP+AP
 1100 CONTINUE
      WRITE(8,1150) 'N(1): ',N(1)
 1150 FORMAT(T3,A,2I5)
      GO TO 9980
 2000 DO 2200 J1 = 1,N(1)
       DO 2100 J2 = 1,N(2)
        AP = P(1,J1)*P(2,J2)
        AX = X(1,J1)+X(2,J2)
        AZ = Z(1,J1)+Z(2,J2)
        IF(AP.LT.POP) FEP = FEP+AP
        IF(AX.GT.XOS) EXP = EXP+AP
        IF(AZ.GT.ZOS) EZP = EZP+AP
 2100  CONTINUE
 2200 CONTINUE
      WRITE(8,2250) 'N(I) FOR I = 1,2: ',N(1),N(2)
 2250 FORMAT(T3,A,2I5)
      GO TO 9980
 3000 DO 3300 J1 = 1,N(1)
       DO 3200 J2 = 1,N(2)
        DO 3100 J3 = 1,N(3)
         AP = P(1,J1)*P(2,J2)*P(3,J3)
         AX = X(1,J1)+X(2,J2)+X(3,J3)
         AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)
         IF(AP.LT.POP) FEP = FEP+AP
         IF(AX.GT.XOS) EXP = EXP+AP
         IF(AZ.GT.ZOS) EZP = EZP+AP
 3100   CONTINUE
 3200  CONTINUE
 3300 CONTINUE
      WRITE(8,3350) 'N(I) FOR I = 1,2,3: ',N(1),N(2),N(3)
 3350 FORMAT(T3,A,3I5)
      GO TO 9980
 4000 DO 4400 J1 = 1,N(1)
       DO 4300 J2 = 1,N(2)
        DO 4200 J3 = 1,N(3)
         DO 4100 J4 = 1,N(4)
          AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)
          AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)
          AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)
          IF(AP.LT.POP) FEP = FEP+AP
          IF(AX.GT.XOS) EXP = EXP+AP
          IF(AZ.GT.ZOS) EZP = EZP+AP
 4100    CONTINUE
 4200   CONTINUE
 4300  CONTINUE
 4400 CONTINUE
      WRITE(8,4450) 'N(I) FOR I = 1,2,3,4: ',N(1),N(2),N(3),N(4)
 4450 FORMAT(T3,A,4I5)
      GO TO 9980
 5000 DO 5500 J1 = 1,N(1)
       DO 5400 J2 = 1,N(2)
        DO 5300 J3 = 1,N(3)
         DO 5200 J4 = 1,N(4)
          DO 5100 J5 = 1,N(5)
           AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)*P(5,J5)
           AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)
           AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)
           IF(AP.LT.POP) FEP = FEP+AP
           IF(AX.GT.XOS) EXP = EXP+AP
           IF(AZ.GT.ZOS) EZP = EZP+AP
 5100     CONTINUE
 5200    CONTINUE
 5300   CONTINUE
 5400  CONTINUE
 5500 CONTINUE
      WRITE(8,5550) 'N(I) FOR I = 1,2,3,4,5: ',N(1),N(2),N(3),N(4),N(5)
 5550 FORMAT(T3,A,5I5)
      GO TO 9980
 6000 DO 6600 J1 = 1,N(1)
       DO 6500 J2 = 1,N(2)
        DO 6400 J3 = 1,N(3)
         DO 6300 J4 = 1,N(4)
          DO 6200 J5 = 1,N(5)
           DO 6100 J6 = 1,N(6)
            AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)*P(5,J5)*P(6,J6)
            AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
            AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
            IF(AP.LT.POP) FEP = FEP+AP
            IF(AX.GT.XOS) EXP = EXP+AP
            IF(AZ.GT.ZOS) EZP = EZP+AP
 6100      CONTINUE
 6200     CONTINUE
 6300    CONTINUE
 6400   CONTINUE
 6500  CONTINUE
 6600 CONTINUE
      WRITE(8,6650) 'N(I) FOR I = 1,...,6: ',N(1),N(2),N(3),N(4),N(5),
     &                                       N(6)
 6650 FORMAT(T3,A,6I5)
      GO TO 9980
 7000 DO 7700 J1 = 1,N(1)
       DO 7600 J2 = 1,N(2)
        DO 7500 J3 = 1,N(3)
         DO 7400 J4 = 1,N(4)
          DO 7300 J5 = 1,N(5)
           DO 7200 J6 = 1,N(6)
            DO 7100 J7 = 1,N(7)
             AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)*P(5,J5)*P(6,J6)
     &           *P(7,J7)
             AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &           +X(7,J7)
             AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &           +Z(7,J7)
             IF(AP.LT.POP) FEP = FEP+AP
             IF(AX.GT.XOS) EXP = EXP+AP
             IF(AZ.GT.ZOS) EZP = EZP+AP
 7100       CONTINUE
 7200      CONTINUE
 7300     CONTINUE
 7400    CONTINUE
 7500   CONTINUE
 7600  CONTINUE
 7700 CONTINUE
      WRITE(8,7750) 'N(I) FOR I = 1,...,7: ',N(1),N(2),N(3),N(4),N(5),
     &                                       N(6),N(7)
 7750 FORMAT(T3,A,7I5)
      GO TO 9980
 8000 DO 8800 J1 = 1,N(1)
       DO 8700 J2 = 1,N(2)
        DO 8600 J3 = 1,N(3)
         DO 8500 J4 = 1,N(4)
          DO 8400 J5 = 1,N(5)
           DO 8300 J6 = 1,N(6)
            DO 8200 J7 = 1,N(7)
             DO 8100 J8 = 1,N(8)
              AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)*P(5,J5)*P(6,J6)
     &            *P(7,J7)*P(8,J8)
              AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &            +X(7,J7)+X(8,J8)
              AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &            +Z(7,J7)+Z(8,J8)
              IF(AP.LT.POP) FEP = FEP+AP
              IF(AX.GT.XOS) EXP = EXP+AP
              IF(AZ.GT.ZOS) EZP = EZP+AP
 8100        CONTINUE
 8200       CONTINUE
 8300      CONTINUE
 8400     CONTINUE
 8500    CONTINUE
 8600   CONTINUE
 8700  CONTINUE
 8800 CONTINUE
      WRITE(8,8850) 'N(I) FOR I = 1,...,8: ',N(1),N(2),N(3),N(4),N(5),
     &                                       N(6),N(7),N(8)
 8850 FORMAT(T3,A,8I5)
      GO TO 9980
 9000 DO 9900 J1 = 1,N(1)
       DO 9800 J2 = 1,N(2)
        DO 9700 J3 = 1,N(3)
         DO 9600 J4 = 1,N(4)
          DO 9500 J5 = 1,N(5)
           DO 9400 J6 = 1,N(6)
            DO 9300 J7 = 1,N(7)
             DO 9200 J8 = 1,N(8)
              DO 9100 J9 = 1,N(9)
               AP = P(1,J1)*P(2,J2)*P(3,J3)*P(4,J4)*P(5,J5)*P(6,J6)
     &             *P(7,J7)*P(8,J8)*P(9,J9)
               AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &             +X(7,J7)+X(8,J8)+X(9,J9)
               AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &             +Z(7,J7)+Z(8,J8)+Z(9,J9)
               IF(AP.LT.POP) FEP = FEP+AP
               IF(AX.GT.XOS) EXP = EXP+AP
               IF(AZ.GT.ZOS) EZP = EZP+AP
 9100         CONTINUE
 9200        CONTINUE
 9300       CONTINUE
 9400      CONTINUE
 9500     CONTINUE
 9600    CONTINUE
 9700   CONTINUE
 9800  CONTINUE
 9900 CONTINUE
      WRITE(8,9950) 'N(I) FOR I = 1,...,9: ',N(1),N(2),N(3),N(4),N(5),
     &                                       N(6),N(7),N(8),N(9)
 9950 FORMAT(T3,A,9I5)
 9980 WRITE(8,9990) 'COMBINED FISHER''S EXACT TEST P-VALUE = ',FEP
 9990 FORMAT(T3,A,D20.13)
      WRITE(8,9990) 'COMBINED EXACT CHI-SQUARE TEST P-VALUE = ',EXP
      WRITE(8,9990) 'COMBINED EXACT LOG-LR TEST P-VALUE = ',EZP
      END
