      PROGRAM EHOSMP
      IMPLICIT REAL*8(A-H,O-Z)
C
C    ONE-SAMPLE OPTION: (KW = 1)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR AN EXACT ONE-SAMPLE MULTIVARIATE HOTELLING'S TEST WITH
C    HYPOTHESIZED MEAN VECTOR OF THE R RESPONSES UNDER THE MULTIVARIATE
C    NORMALITY ASSUMPTIONS.  THE PRESENT MAXIMUM VALUES OF B AND
C    R IN THIS PROGRAM ARE 30 AND 25, RESPECTIVELY.
C
C    THE DATA MATRIX FOR ONE-SAMPLE OPTION MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES B, R AND KW = 1
C    THE THIRD LINE IS THE HYPOTHESIZED R MEAN  RESPONSE VALUES
C        U(1),    U(2),    ...,U(R)
C    THE JTH OBJECT'S R RESPONSE VALUES ARE THEN ON THE (3+J)TH LINE
C    WHERE J = 1,...,B (THE # OF OBJECTS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C        A(1,2,1),A(1,2,2),...,A(1,2,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, B IS # OF BLOCKS
C    (IN THIS CASE OBJECTS), AND R IS # OF RESPONSES.
C    NOTE: ASSOCIATE B AND R WITH KB AND KR IN THIS PROGRAM.
C
C
C    MATCHED-PAIRS OPTION: (KW = 2)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR A MATCHED-PAIRS MULTIVARIATE HOTELLING'S TEST
C    UNDER THE MULTIVARIATE NORMALITY ASSUMPTIONS.  THE PRESENT
C    MAXIMUM VALUES OF B AND R IN THIS PROGRAM ARE 30 AND 25, RESPECTIVELY.
C
C    THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES B, R AND KW = 2
C    THE JTH PAIR'S 1ST R RESPONSE VALUES ARE ON THE (2+J)TH LINE
C    AND THE 2ND R RESPONSE VALUES ARE ON THE (2+B+J)TH LINE WHERE
C    J = 1,...,B (THE # OF PAIRS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C        A(2,1,1),A(2,1,2),...,A(2,1,R)
C                 ...
C        A(2,B,1),A(2,B,2),...,A(2,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, B IS # OF BLOCKS
C    (IN THIS CASE PAIRS), AND R IS # OF RESPONSES.
C    NOTE: ASSOCIATE B AND R WITH KB AND KR IN THIS PROGRAM.
C
C    PROGRAM MODIFIED 3/25/95
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  U(25),X(25),DIFF(30),W(25,25),Z(30,25),RAW(2,30,25)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,2) FIN
    2 FORMAT(A)
      WRITE(*,*) 'ENTER: FILE NAME FOR OUTPUT RESULTS'
      READ(*,2) FOUT
      OPEN(5,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      READ(5,2) HEAD
      READ(5,*) KB,KR,KW
      IF(KB.LE.KR) STOP 'B LESS THAN OR EQUAL TO R'
      IF(KW.EQ.2) GO TO 21
      READ(5,*) (U(K),K=1,KR)
      DO 4 J = 1,KB
         READ(5,*,END=170) (RAW(1,J,K),K=1,KR)
    4 CONTINUE
      CLOSE(5)
      WRITE(6,2) HEAD
      WRITE(6,5) KR,KB
    5 FORMAT('ONE-SAMPLE MULTI-VAR HOTELLING EXACT PERMUTATION TEST:'
     1    /4X,'WITH ',I3,' RESPONSES AND ',I3,' BLOCKS.')
      DO 20 J = 1,KB
         DO 16 K = 1,KR
            DIFF(K) = RAW(1,J,K)-U(K)
   16    CONTINUE
         DO 18 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   18    CONTINUE
   20 CONTINUE
      GO TO 65
   21 DO 23 I = 1,2
         DO 22 J = 1,KB
            READ(5,*,END=170) (RAW(I,J,K),K=1,KR)
   22    CONTINUE
   23 CONTINUE
      CLOSE(5)
      WRITE(6,2) HEAD
      WRITE(6,25) KR,KB
   25 FORMAT('MATCHED-PAIRS MULTI-VAR HOTELLING EXACT PERMUTATION TEST:'
     1    /4X,'WITH ',I3,' RESPONSES AND ',I3,' BLOCKS.')
      DO 60 J = 1,KB
         DO 56 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   56    CONTINUE
         DO 58 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   58    CONTINUE
   60 CONTINUE
   65 DO 68 J = 1,KB
         DO 67 K = 1,KR
            Z(J,K) = RAW(1,J,K)
   67    CONTINUE
   68 CONTINUE
      DO 70 I = 1,KR
         X(I) = ZERO
         DO 69 J = 1,KB
            X(I) = X(I)+RAW(1,J,I)
   69    CONTINUE
         X(I) = X(I)/KB
   70 CONTINUE
      DO 80 I = 1,KR
         DO 79 J = I,KR
            W(I,J) = ZERO
            DO 78 K = 1,KB
               W(I,J) = W(I,J)+(RAW(1,K,I)-X(I))*(RAW(1,K,J)-X(J))
   78       CONTINUE
            W(I,J) = W(I,J)/KB
            W(J,I) = W(I,J)
   79    CONTINUE
   80 CONTINUE
      CALL INVERT(KR,W)
      TSQUARE = ZERO
      DO 85 I = 1,KR
         DO 84 J = 1,KR
            TSQUARE = TSQUARE+X(I)*W(I,J)*X(J)
   84    CONTINUE
   85 CONTINUE
      T1 = TSQUARE*(ONE-1.0D-12)
C
      DN = ONE*KR
      DD = ONE*(KB-KR)
      ADJTSQ = TSQUARE*DD/DN
      TSQUARE = TSQUARE*(KB-1)
      WRITE(6,2) ' '
      WRITE(6,160) '             OBSERVED TSQUARE = ',TSQUARE
  160 FORMAT(T8,A,G16.7)
      WRITE(6,160) '                       ADJTSQ = ',ADJTSQ
      CALL FVALUE(ADJTSQ,DN,DD,PTSQ)
      WRITE(6,160) 'P(TSQUARE > OBSERVED TSQUARE) = ',PTSQ
      CALL COMB(KB,KR,KOUNT1,KOUNT2,T1,Z)
      EPV = ONE*KOUNT2/KOUNT1
      WRITE(6,165) 'NUMBER OF TSQUARES  OBSERVED TSQUARE = ',KOUNT2
  165 FORMAT(T8,A,I8)
      WRITE(6,165) 'TOTAL NUMBER OF TSQUARE VALUES = ',KOUNT1
      WRITE(6,160) 'EXACT P-VALUE OF OBSERVED TSQUARE = ',EPV
      GO TO 200
  170 WRITE(6,180)
  180 FORMAT(' E R R O R: END-OF-FILE ENCOUNTERED.')
      WRITE(6,190) KB,KR
  190 FORMAT(' CHECK THESE:  B = ',I3,' R = ',I3)
  200 END
C
C
      SUBROUTINE INVERT(N,W)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  KP(25),KS(25),KC(25),W(25,25)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IS = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IS.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IS,L)
               W(IS,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KS(I) = IS
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KS(J).NE.KC(J)) THEN
            K = KS(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE FVALUE(ADJTSQ,DN,DD,PTSQ)
      IMPLICIT REAL*8(A-H,O-Z)
C
      A = DN/2
      B = DD/2
      Y = A*ADJTSQ/(A*ADJTSQ+B)
      PTSQ = 1.0D0-BETAIN(Y,A,B)
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION BETAIN(X,A,B)
C
      DOUBLE PRECISION  X,A,B,ONE,TOL,AB,CX,XX,AA,BB,TERM,AI,RX,TEMP,
     +       DLNBTA
      LOGICAL  INDEX
C
      ONE = 1.0D0
      TOL = 0.1D-7
      BETAIN = X
      AB = A+B
      CX = ONE-X
      IF (A.GE.AB*X) GO TO 10
      XX = CX
      CX = X
      AA = B
      BB = A
      INDEX = .TRUE.
      GO TO 20
   10 XX = X
      AA = A
      BB = B
      INDEX = .FALSE.
   20 TERM = ONE
      AI = ONE
      BETAIN = ONE
      NS = IDINT(BB+CX*AB)
      RX = XX/CX
   30 TEMP = BB-AI
      IF(NS.EQ.0) RX = XX
   40 TERM = TERM*TEMP*RX/(AA+AI)
      BETAIN = BETAIN+TERM
      TEMP = DABS(TERM)
      IF(TEMP.LE.TOL.AND.TEMP.LE.TOL*BETAIN) GO TO 50
      AI = AI+1
      NS = NS-1
      IF(NS.GE.0) GO TO 30
      TEMP = AB
      AB = AB+1
      GO TO 40
   50 BETAIN = BETAIN*DEXP(AA*DLOG(XX)+(BB-1)*DLOG(CX)-
     +         DLNBTA(AA,BB))/AA
      IF(INDEX) BETAIN = ONE-BETAIN
      END
C
C
      DOUBLE PRECISION FUNCTION DLNBTA(A,B)
C
      DOUBLE PRECISION  A,B,DLNGAM
C
      DLNBTA = DLNGAM(A)+DLNGAM(B)-DLNGAM(A+B)
      END
C
C
      DOUBLE PRECISION FUNCTION DLNGAM(X)
C
      DOUBLE PRECISION  X,ONE,Z,D,PI,F
C
      ONE  = 1.0D0
      Z  = X-1
      D  = ONE
      PI = DATAN(ONE)*4
      F  = Z+10
      DO 10 J = 1,9
         D = D*(Z+J)
   10 CONTINUE
      DLNGAM = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2
     +         -DLOG(D)+ONE/(F*12)-ONE/(F**3*360)
     +         +ONE/(F**5*1260)-ONE/(F**7*1680)
     +         +ONE/(F**9*1188)
      END
C
C
      SUBROUTINE COMB(KB,KR,KOUNT1,KOUNT2,T1,Z)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  M(30),D(30,25),Z(30,25)
C
      ONE = 1.0D0
      DO 10 I = 1,KB
         M(I) = 1
   10 CONTINUE
      KOUNT1 = 0
      KOUNT2 = 0
      KOUNT3 = 0
      NMINUS = 0
      LAST = 0
      NN = 100
      JJ = 1
      GO TO 50
   20 J = 0
   30 J = J+1
      IF(MOD(KOUNT3,2).EQ.1) GO TO 40
      KOUNT3 = KOUNT3/2
      GO TO 30
   40 LAST = M(J)
      M(J) = -LAST
   50 KOUNT1 = KOUNT1+1
      IF(KOUNT1.EQ.NN*JJ) THEN
         WRITE(*,*) KOUNT1
         JJ = JJ+1
      END IF
      KOUNT3 = KOUNT1
      NMINUS = NMINUS+LAST
      DO 56 J = 1,KB
         DO 54 K = 1,KR
            D(J,K) = Z(J,K)
   54    CONTINUE
   56 CONTINUE
      CALL XACT(KB,KR,KOUNT2,T1,D,M)
      IF(NMINUS.EQ.1.AND.M(KB).EQ.-1) RETURN
      GO TO 20
      END
C
C
      SUBROUTINE XACT(KB,KR,KOUNT2,T1,D,M)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  M(30),X(25),D(30,25),W(25,25)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DO 20 J = 1,KR
         DO 10 I = 1,KB
            D(I,J) = M(I)*D(I,J)
   10    CONTINUE
   20 CONTINUE
      DO 70 I = 1,KR
         X(I) = ZERO
         DO 69 J = 1,KB
            X(I) = X(I)+D(J,I)
   69    CONTINUE
         X(I) = X(I)/KB
   70 CONTINUE
      DO 80 I = 1,KR
         DO 79 J = I,KR
            W(I,J) = ZERO
            DO 78 K = 1,KB
               W(I,J) = W(I,J)+(D(K,I)-X(I))*(D(K,J)-X(J))
   78       CONTINUE
            W(I,J) = W(I,J)/KB
            W(J,I) = W(I,J)
   79    CONTINUE
   80 CONTINUE
      CALL INVERT(KR,W)
      T = ZERO
      DO 85 I = 1,KR
         DO 84 J = 1,KR
            T = T+X(I)*W(I,J)*X(J)
   84    CONTINUE
   85 CONTINUE
      IF(T.GE.T1) KOUNT2 = KOUNT2+1
      RETURN
      END

