      PROGRAM EXGF
C
C     PROGRAM EXGF IS A DRIVER PROGRAM, CALLING A SELECTED SUBROUTINE
C     TO COMPUTE FISHER'S EXACT PROBABILITY FOR GOODNESS-OF-FIT
C     CLASSIFICATIONS WITH 2, 3, 4, 5 AND 6 CELLS.  THE CONSTANTS
C     (INIT AND EPSLN) ARE MACHINE-DEPENDENT VALUES TO COMPENSATE
C     FOR ROUNDING (EPSLN) AND TO PREVENT UNDER/OVER FLOW (INIT).
C     THESE VALUES MAY BE CHANGED TO ACCOMMODATE SPECIFIC MACHINES.
C
      INTEGER  CHOICE,M1,M2,M3,M4,M5,M6
      REAL*8   P,PV1,PV2,PV3,P1,P2,P3,P4,P5,INIT,EPSLN
      PARAMETER (INIT = 1.0D-150, EPSLN = 1.0D-12)
C
    1 WRITE  (*,2)
    2 FORMAT (///,T22,'FISHER EXACT PROBABILITY TESTS',
     &         //,T15,'(1)  FISHER EXACT TEST FOR 2 CELLS',
     &          /,T15,'(2)  FISHER EXACT TEST FOR 3 CELLS',
     &          /,T15,'(3)  FISHER EXACT TEST FOR 4 CELLS',
     &          /,T15,'(4)  FISHER EXACT TEST FOR 5 CELLS',
     &          /,T15,'(5)  FISHER EXACT TEST FOR 6 CELLS',
     &          /,T15,'(6)  EXIT FROM THE FISHER EXACT PROGRAM',
     &         //,T15,'     PLEASE CHOOSE A NUMBER FROM 1 TO 6')
      READ   (*,*) CHOICE
      IF(CHOICE.LT.1.OR.CHOICE.GT.6) STOP 'NUMBER NOT FROM 1 TO 6'
      GO TO (3,10,11,12,13,15) CHOICE
C
C     COMPUTE EXACT GOODNESS-OF-FIT TEST FOR 2 CELLS
C
    3 WRITE  (*,4) 'FISHER''S EXACT GOODNESS-OF-FIT TEST FOR 2 CELLS'
    4 FORMAT (/,T2,A,/)
      WRITE  (*,5) 'ENTER: M1,M2,P1'
    5 FORMAT (T2,A)
      READ   (*,*)  M1,M2,P1
      CALL EXGF2 (M1,M2,P1,INIT,EPSLN,PV1,PV2,PV3,MX)
      IF(MX.EQ.1) THEN
         WRITE (*,5) 'THE ONE-SIDED P-VALUE IS LEFT TAILED'
      ELSE
         WRITE (*,5) 'THE ONE-SIDED P-VALUE IS RIGHT TAILED'
      END IF
      WRITE  (*,8) 'FISHER EXACT 2-CELL ONE-SIDED P-VALUE = ',PV1
    8 FORMAT (/,T5,A,G14.6E3)
      WRITE  (*,9) '1 + P(OBSERVED #) - (1-SIDED P-VALUE) = ',PV2
    9 FORMAT (T5,A,G14.6E3)
      WRITE  (*,9) 'FISHER EXACT 2-CELL TWO-SIDED P-VALUE = ',PV3
      GO TO 1
C
C     COMPUTE EXACT GOODNESS-OF-FIT TEST FOR 3 CELLS
C
   10 WRITE  (*,4) 'FISHER''S EXACT GOODNESS-OF-FIT TEST FOR 3 CELLS'
      WRITE  (*,5) 'ENTER: M1,M2,M3,P1,P2'
      READ   (*,*)  M1,M2,M3,P1,P2
      CALL EXGF3 (M1,M2,M3,P1,P2,INIT,EPSLN,P)
      WRITE  (*,8) 'FISHER EXACT 3-CELL PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT GOODNESS-OF-FIT TEST FOR 4 CELLS
C
   11 WRITE  (*,4) 'FISHER''S EXACT GOODNESS-OF-FIT TEST FOR 4 CELLS'
      WRITE  (*,5) 'ENTER: M1,M2,M3,M4,P1,P2,P3'
      READ   (*,*)  M1,M2,M3,M4,P1,P2,P3
      CALL EXGF4 (M1,M2,M3,M4,P1,P2,P3,INIT,EPSLN,P)
      WRITE  (*,8) 'FISHER EXACT 4-CELL PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT GOODNESS-OF-FIT TEST FOR 5 CELLS
C
   12 WRITE  (*,4) 'FISHER''S EXACT GOODNESS-OF-FIT TEST FOR 5 CELLS'
      WRITE  (*,5) 'ENTER: M1,M2,M3,M4,M5,P1,P2,P3,P4'
      READ   (*,*)  M1,M2,M3,M4,M5,P1,P2,P3,P4
      CALL EXGF5 (M1,M2,M3,M4,M5,P1,P2,P3,P4,INIT,EPSLN,P)
      WRITE  (*,8) 'FISHER EXACT 5-CELL PROBABILITY VALUE = ',P
      GO TO 1
C
C     COMPUTE EXACT GOODNESS-OF-FIT TEST FOR 6 CELLS
C
   13 WRITE  (*,4) 'FISHER''S EXACT GOODNESS-OF-FIT TEST FOR 6 CELLS'
      WRITE  (*,5) 'ENTER: M1,M2,M3,M4,M5,M6,P1,P2,P3,P4,P5'
      READ   (*,*)  M1,M2,M3,M4,M5,M6,P1,P2,P3,P4,P5
      CALL EXGF6 (M1,M2,M3,M4,M5,M6,P1,P2,P3,P4,P5,INIT,EPSLN,P)
      WRITE  (*,8) 'FISHER EXACT 6-CELL PROBABILITY VALUE = ',P
      GO TO 1
C
C     EXIT THE FISHER EXACT TEST PROGRAM
C
   15 WRITE  (*,16) 'FISHER EXACT PROGRAM IS TERMINATED'
   16 FORMAT (T20,A,/)
      END
C
C
C
      SUBROUTINE EXGF2 (M1,M2,P1,INIT,EPSLN,PV1,PV2,PV3,MX)
C
      INTEGER  I,M1,M2,MX,N
      REAL*8   ONE,ZERO,P1,PV1,PV2,PV3,A1,U1,U2,U3,H,T,S1,S2,
     &         INIT,EPSLN,W
C
      ONE = 1.0D0
      ZERO = 0.0D0
      H = INIT
      U2  = INIT
      T = INIT
      S1 = ZERO
      S2 = ZERO
      U1 = ZERO
      U3 = ZERO
      N = M1+M2
      MX = 0
      IF(P1.LE.ZERO.OR.P1.GE.ONE)  STOP '0 < P1 < 1 IS NOT SATISFIED'
      A1 = P1/(ONE-P1)
      IF(M1.EQ.0) W = U2
      IF(M1.GT.0) THEN
        DO 10 I = 0,M1-1
          U2 = U2*A1*(N-I)/(I+1)
          IF(I.EQ.M1-1) W = U2
   10   CONTINUE
        U1 = U2*M1/(A1*(N-M1+1))
      ENDIF
      IF(M1.LT.N) U3 = U2*A1*(N-M1)/(M1+1)
      U2 = U2*(ONE+EPSLN)
      IF(U1.LE.U3) THEN
         S1 = H
         MX = 1
      END IF
      IF(H.LE.U2) S2 = H
      DO 20 I = 0,N-1
        H = H*A1*(N-I)/(I+1)
        T = T+H
        IF(U1.LE.U3.AND.I.LE.M1-1.OR.U1.GT.U3.AND.I.GE.M1-1) S1 = S1+H
        IF(H.LE.U2) S2 = S2+H
   20 CONTINUE
      PV1 = S1/T
      PV2 = ONE+(W/T)-PV1
      PV3 = S2/T
      RETURN
      END
C
C
C
      SUBROUTINE EXGF3  (M1,M2,M3,P1,P2,INIT,EPSLN,P)
C
      INTEGER  I,J,M1,M2,M3,N
      REAL*8   ONE,ZERO,P,P1,P2,P3,A1,A2,U,H,S,T,W1,INIT,EPSLN
C
      ONE = 1.0D0
      ZERO = 0.0D0
      H = INIT
      T = INIT
      U = INIT
      S = ZERO
      W1 = ZERO
      N = M1+M2+M3
      P3 = ONE-P1-P2
      IF(DMIN1(P1,P2,P3).LE.ZERO)  STOP 'P1, P2 OR P3 IS < OR = 0'
      A1 = P1/P3
      A2 = P2/P3
      IF(M1.GT.0) THEN
        DO 10 I = 0,M1-1
          U = U*A1*(N-I)/(I+1)
   10   CONTINUE
      ENDIF
      IF(M2.GT.0) THEN
        DO 20 J = 0,M2-1
          U = U*A2*(N-M1-J)/(J+1)
   20   CONTINUE
      ENDIF
      U = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
      DO 40 I = 0,N-1
        W1 = H
        DO 30 J = 0,N-I-1
          H = H*A2*(N-I-J)/(J+1)
          T = T+H
          IF(H.LE.U) S = S+H
   30   CONTINUE
        H = W1*A1*(N-I)/(I+1)
        T = T+H
        IF(H.LE.U) S = S+H
   40 CONTINUE
      P = S/T
      RETURN
      END
C
C
C
      SUBROUTINE EXGF4  (M1,M2,M3,M4,P1,P2,P3,INIT,EPSLN,P)
C
      INTEGER  I,J,K,M1,M2,M3,N
      REAL*8   ONE,ZERO,P,P1,P2,P3,P4,A1,A2,A3,U,H,S,T,W1,W2,
     &         INIT,EPSLN
C
      ONE = 1.0D0
      ZERO = 0.0D0
      H = INIT
      T = INIT
      U = INIT
      S = ZERO
      W1 = ZERO
      W2 = ZERO
      N = M1+M2+M3+M4
      P4 = ONE-P1-P2-P3
      IF(DMIN1(P1,P2,P3,P4).LE.ZERO) STOP 'P1, P2, P3 OR P4 IS < OR = 0'
      A1 = P1/P4
      A2 = P2/P4
      A3 = P3/P4
      IF(M1.GT.0) THEN
        DO 10 I = 0,M1-1
          U = U*A1*(N-I)/(I+1)
   10   CONTINUE
      ENDIF
      IF(M2.GT.0) THEN
        DO 20 J = 0,M2-1
          U = U*A2*(N-M1-J)/(J+1)
   20   CONTINUE
      ENDIF
      IF(M3.GT.0) THEN
        DO 30 K = 0,M3-1
          U = U*A3*(N-M1-M2-K)/(K+1)
   30   CONTINUE
      ENDIF
      U = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
      DO 60 I = 0,N-1
        W1 = H
        DO 50 J = 0,N-I-1
          W2 = H
          DO 40 K = 0,N-I-J-1
            H = H*A3*(N-I-J-K)/(K+1)
            T = T+H
            IF(H.LE.U) S = S+H
   40     CONTINUE
          H = W2*A2*(N-I-J)/(J+1)
          T = T+H
          IF(H.LE.U) S = S+H
   50   CONTINUE
        H = W1*A1*(N-I)/(I+1)
        T = T+H
        IF(H.LE.U) S = S+H
   60 CONTINUE
      P = S/T
      RETURN
      END
C
C
C
      SUBROUTINE EXGF5  (M1,M2,M3,M4,M5,P1,P2,P3,P4,INIT,EPSLN,P)
C
      INTEGER  I,J,K,L,M1,M2,M3,M4,M5,N
      REAL*8   ONE,ZERO,P,P1,P2,P3,P4,P5,A1,A2,A3,A4,U,H,S,T,
     &         W1,W2,W3,INIT,EPSLN
C
      ONE = 1.0D0
      ZERO = 0.0D0
      H = INIT
      T = INIT
      U = INIT
      S = ZERO
      W1 = ZERO
      W2 = ZERO
      W3 = ZERO
      N = M1+M2+M3+M4+M5
      P5 = ONE-P1-P2-P3-P4
      IF(DMIN1(P1,P2,P3,P4,P5).LE.ZERO) STOP
     &       'P1, P2, P3, P4 OR P5 IS < OR = 0'
      A1 = P1/P5
      A2 = P2/P5
      A3 = P3/P5
      A4 = P4/P5
      IF(M1.GT.0) THEN
        DO 10 I = 0,M1-1
          U = U*A1*(N-I)/(I+1)
   10   CONTINUE
      ENDIF
      IF(M2.GT.0) THEN
        DO 20 J = 0,M2-1
          U = U*A2*(N-M1-J)/(J+1)
   20   CONTINUE
      ENDIF
      IF(M3.GT.0) THEN
        DO 30 K = 0,M3-1
          U = U*A3*(N-M1-M2-K)/(K+1)
   30   CONTINUE
      ENDIF
      IF(M4.GT.0) THEN
        DO 40 L = 0,M4-1
          U = U*A4*(N-M1-M2-M3-L)/(L+1)
   40   CONTINUE
      ENDIF
      U = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
      DO 80 I = 0,N-1
        W1 = H
        DO 70 J = 0,N-I-1
          W2 = H
          DO 60 K = 0,N-I-J-1
            W3 = H
            DO 50 L = 0,N-I-J-K-1
              H = H*A4*(N-I-J-K-L)/(L+1)
              T = T+H
              IF(H.LE.U) S = S+H
   50       CONTINUE
            H = W3*A3*(N-I-J-K)/(K+1)
            T = T+H
            IF(H.LE.U) S = S+H
   60     CONTINUE
          H = W2*A2*(N-I-J)/(J+1)
          T = T+H
          IF(H.LE.U) S = S+H
   70   CONTINUE
        H = W1*A1*(N-I)/(I+1)
        T = T+H
        IF(H.LE.U) S = S+H
   80 CONTINUE
      P = S/T
      RETURN
      END
C
C
C
      SUBROUTINE EXGF6  (M1,M2,M3,M4,M5,M6,P1,P2,P3,P4,P5,INIT,EPSLN,P)
C
      INTEGER  I,J,K,L,M,M1,M2,M3,M4,M5,M6,N
      REAL*8   ONE,ZERO,P,P1,P2,P3,P4,P5,P6,A1,A2,A3,A4,A5,
     &         U,H,S,T,W1,W2,W3,W4,INIT,EPSLN
C
      ONE = 1.0D0
      ZERO = 0.0D0
      H = INIT
      T = INIT
      U = INIT
      S = ZERO
      W1 = ZERO
      W2 = ZERO
      W3 = ZERO
      W4 = ZERO
      N = M1+M2+M3+M4+M5+M6
      P6 = ONE-P1-P2-P3-P4-P5
      IF(DMIN1(P1,P2,P3,P4,P5,P6).LE.ZERO) STOP
     &       'P1, P2, P3, P4, P5 OR P6 IS < OR = 0'
      A1 = P1/P6
      A2 = P2/P6
      A3 = P3/P6
      A4 = P4/P6
      A5 = P5/P6
      IF(M1.GT.0) THEN
        DO 10 I = 0,M1-1
          U = U*A1*(N-I)/(I+1)
   10   CONTINUE
      ENDIF
      IF(M2.GT.0) THEN
        DO 20 J = 0,M2-1
          U = U*A2*(N-M1-J)/(J+1)
   20   CONTINUE
      ENDIF
      IF(M3.GT.0) THEN
        DO 30 K = 0,M3-1
          U = U*A3*(N-M1-M2-K)/(K+1)
   30   CONTINUE
      ENDIF
      IF(M4.GT.0) THEN
        DO 40 L = 0,M4-1
          U = U*A4*(N-M1-M2-M3-L)/(L+1)
   40   CONTINUE
      ENDIF
      IF(M5.GT.0) THEN
        DO 50 M = 0,M5-1
          U = U*A5*(N-M1-M2-M3-M4-M)/(M+1)
   50   CONTINUE
      ENDIF
      U = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
      DO 100 I = 0,N-1
        W1 = H
        DO 90 J = 0,N-I-1
          W2 = H
          DO 80 K = 0,N-I-J-1
            W3 = H
            DO 70 L = 0,N-I-J-K-1
              W4 = H
              DO 60 M = 0,N-I-J-K-L-1
                H = H*A5*(N-I-J-K-L-M)/(M+1)
                T = T+H
                IF(H.LE.U) S = S+H
   60         CONTINUE
              H = W4*A4*(N-I-J-K-L)/(L+1)
              T = T+H
              IF(H.LE.U) S = S+H
   70       CONTINUE
            H = W3*A3*(N-I-J-K)/(K+1)
            T = T+H
            IF(H.LE.U) S = S+H
   80     CONTINUE
          H = W2*A2*(N-I-J)/(J+1)
          T = T+H
          IF(H.LE.U) S = S+H
   90   CONTINUE
        H = W1*A1*(N-I)/(I+1)
        T = T+H
        IF(H.LE.U) S = S+H
  100 CONTINUE
      P = S/T
      RETURN
      END
