      PROGRAM FECPV
      IMPLICIT REAL*8(A-H,O-Z)
C
C  PROGRAM FECPV IMPLEMENTS THE DISCRETE DATA METHOD FOR OBTAINING AN EXACT
C  P-VALUE ANALOGOUS TO FISHER'S CONTINUOUS DATA METHOD FOR COMBINING P-VALUES
C  WHEN TWO OR MORE TWO OR MORE TESTS ASSOCIATED WITH DISCRETE P-VALUES
C  ARE COMBINED.  LET N(I) BE THE NUMBER OF DISCRETE FREQUENCIES ASSOCIATED
C  WITH THE ITH OF K TESTS WHERE K = 1,...,9 (THE PROGRAM IS EASILY EXTENDED
C  TO K > 9 WITH TRIVIAL PROGRAMMING IF THIS IS EVER NEEDED).  PROGRAM FECPV
C  HANDLES THE SINGLE TEST CASE WHEN K = 1.  THE PROGRAM ADDRESSES VERY LARGE
C  DATA SETS BY USING EULER'S PARTITIONING REDUCTION WHICH IS POSSIBLE SINCE
C  THE PROBABILITIES OF EACH CATEGORY ASSOCAITED WITH EACH TEST ARE ASSUMED
C  TO BE EQUAL FOR APPLICATIONS SUCH AS LOG-LINEAR MODEL ANALYSES.  THE
C  EXACT P-VALUES OF THIS PROGRAM INCLUDE FISHER'S EXACT, EXACT CHI-SQUARE
C  AND EXACT LOG-LR TESTS.  THE PROGRAM CAN BE EASILY MODIFIED TO HANDLE
C  A WIDE VARIETY OF CASES INVOLVING MULTIPLE TESTS.  THE K TESTS OF THIS
C  PROGRAM MUST BE INDEPENDENT OF ONE ANOTHER.  AN OBVIOUS RESTRICTION OF
C  THIS METHOD IS THAT M = N(1)*...*N(K) CAN NOT BE TOO LARGE SINCE IT IS
C  ALL TOO COMMON THAT EVEN A SINGLE EXACT TEST'S P-VALUE MAY BE BASED ON
C  A VALUE OF N(1) THAT PROHIBITS ITS USE.  FOR EACH TEST, THE FIRST LINE
C  OF THE DATA ENTRY CONSISTS OF THE NUMBER OF EULER TYPE PARTITIONS, THE
C  TOTAL FREQUENCY, THE NUMBER OF CATEGORIES, THE OBSERVED SUM OF SQUARED
C  FREQUENCY VALUES FOR CHI-SQUARE TEST (X), THE OBSERVED SUM OF FREQUENCY
C  TIMES NATURAL LOGARITHM OF FREQUENCY VALUES FOR LOG-LR TEST (Z), THE
C  MULTINOMIAL COEFFICIENT ASSOCIATED THE OBSERVED EULER TYPE PARTITION (F),
C  AND THE OBSERVED ORDINARY MULTINOMIAL COEFFICIENT (W).  THE SUBSEQUENT
C  LIST OF VALUES ARE THE SUM OF SQUARED FREQUENCY VALUE, THE SUM OF FREQUENCY
C  TIMES NATURAL LOGARITHM FREQUENCY VALUE, AND THE MULTINOMIAL COEFFICIENT
C  (F) ASSOCIATED WITH EACH EULER TYPE PARTITION OF THE SEQUENTIALLY LISTED
C  TEST, AND THE ORDINARY MULTINOMIAL COEFFICIENT (W) FOR THE SEQUENTIALLY
C  LISTED TEST.  IF C(I) = 0 OR 1, THEN N(I) = 1 WHICH IS NOT INFORMATIVE.
C  NOTE THAT C(I) IS THE NUMBER OF ITEMS AND L(I) IS THE NUMBER OF PARTITIONS
C  ASSOCIATED WITH THE ITH OF K TESTS (I = 1,...,K).
C
      CHARACTER FIN*40,FOUT*40
      DIMENSION N(9),C(9),L(9),XO(9),ZO(9),FO(9),WO(9),
     &          X(9,150),Z(9,150),F(9,150),W(9,150)
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'THE NUMBER OF INDEPENDENT TESTS:'
      READ(*,*) K
      IF(K.GT.9) STOP 'K IS GREATER THAN 9'
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,100) FOUT
      OPEN(1,FILE=FIN)
      OPEN(2,FILE=FOUT,STATUS='NEW')
      DO 200 I = 1,K
         READ(1,*) N(I),C(I),L(I),XO(I),ZO(I),FO(I),WO(I)
         XO(I) = XO(I)*L(I)/C(I)-C(I)
         ZO(I) = (ZO(I)-C(I)*DLOG(C(I)/L(I)))*2
         DO 150 J = 1,N(I)
            READ(1,*) X(I,J),Z(I,J),F(I,J),W(I,J)
            X(I,J) = X(I,J)*L(I)/C(I)-C(I)
            Z(I,J) = (Z(I,J)-C(I)*DLOG(C(I)/L(I)))*2
  150    CONTINUE
  200 CONTINUE
      AOP = ONE
      XOS = ZERO
      ZOS = ZERO
      FEP = ZERO
      EXP = ZERO
      EZP = ZERO
      TOP = ZERO
      TOT = ZERO
      M = 1
      DO 300 I = 1,K
         AOP = AOP*FO(I)*WO(I)
         XOS = XOS+XO(I)
         ZOS = ZOS+ZO(I)
         M = M*N(I)
  300 CONTINUE
      WRITE(2,400) 'TOTAL NUMBER OF LOOPS = ',M
  400 FORMAT(T3,A,I12)
      AOP = AOP*(ONE+1.0D-11)
      XOS = XOS*(ONE-1.0D-12)
      ZOS = ZOS*(ONE-1.0D-12)
      IF(K.EQ.1) GO TO 1000
      IF(K.EQ.2) GO TO 2000
      IF(K.EQ.3) GO TO 3000
      IF(K.EQ.4) GO TO 4000
      IF(K.EQ.5) GO TO 5000
      IF(K.EQ.6) GO TO 6000
      IF(K.EQ.7) GO TO 7000
      IF(K.EQ.8) GO TO 8000
      IF(K.EQ.9) GO TO 9000
 1000 DO 1100 J1 = 1,N(1)
       AF = F(1,J1)
       AP = AF*W(1,J1)
       AX = X(1,J1)
       AZ = Z(1,J1)
       IF(AP.LT.AOP) FEP = FEP+AP
       IF(AX.GT.XOS) EXP = EXP+AP
       IF(AZ.GT.ZOS) EZP = EZP+AP
       TOT = TOT+AP
       TOP = TOP+AF
 1100 CONTINUE
      WRITE(2,1105) 'N(I) FOR I = 1: ',N(1)
 1105 FORMAT(T3,A,I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 2000 DO 2200 J1 = 1,N(1)
       DO 2100 J2 = 1,N(2)
        AF = F(1,J1)*F(2,J2)
        AP = AF*W(1,J1)*W(2,J2)
        AX = X(1,J1)+X(2,J2)
        AZ = Z(1,J1)+Z(2,J2)
        IF(AP.LT.AOP) FEP = FEP+AP
        IF(AX.GT.XOS) EXP = EXP+AP
        IF(AZ.GT.ZOS) EZP = EZP+AP
        TOT = TOT+AP
        TOP = TOP+AF
 2100  CONTINUE
 2200 CONTINUE
      WRITE(2,2205) 'N(I) FOR I = 1,2: ',N(1),N(2)
 2205 FORMAT(T3,A,2I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 3000 DO 3300 J1 = 1,N(1)
       DO 3200 J2 = 1,N(2)
        DO 3100 J3 = 1,N(3)
         AF = F(1,J1)*F(2,J2)*F(3,J3)
         AP = AF*W(1,J1)*W(2,J2)*W(3,J3)
         AX = X(1,J1)+X(2,J2)+X(3,J3)
         AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)
         IF(AP.LT.AOP) FEP = FEP+AP
         IF(AX.GT.XOS) EXP = EXP+AP
         IF(AZ.GT.ZOS) EZP = EZP+AP
         TOT = TOT+AP
         TOP = TOP+AF
 3100   CONTINUE
 3200  CONTINUE
 3300 CONTINUE
      WRITE(2,3305) 'N(I) FOR I = 1,2,3: ',N(1),N(2),N(3)
 3305 FORMAT(T3,A,3I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 4000 DO 4400 J1 = 1,N(1)
       DO 4300 J2 = 1,N(2)
        DO 4200 J3 = 1,N(3)
         DO 4100 J4 = 1,N(4)
          AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)
          AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)
          AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)
          AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)
          IF(AP.LT.AOP) FEP = FEP+AP
          IF(AX.GT.XOS) EXP = EXP+AP
          IF(AZ.GT.ZOS) EZP = EZP+AP
          TOT = TOT+AP
          TOP = TOP+AF
 4100    CONTINUE
 4200   CONTINUE
 4300  CONTINUE
 4400 CONTINUE
      WRITE(2,4405) 'N(I) FOR I = 1,2,3,4: ',N(1),N(2),N(3),N(4)
 4405 FORMAT(T3,A,4I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 5000 DO 5500 J1 = 1,N(1)
       DO 5400 J2 = 1,N(2)
        DO 5300 J3 = 1,N(3)
         DO 5200 J4 = 1,N(4)
          DO 5100 J5 = 1,N(5)
           AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)*F(5,J5)
           AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)*W(5,J5)
           AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)
           AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)
           IF(AP.LT.AOP) FEP = FEP+AP
           IF(AX.GT.XOS) EXP = EXP+AP
           IF(AZ.GT.ZOS) EZP = EZP+AP
           TOT = TOT+AP
           TOP = TOP+AF
 5100     CONTINUE
 5200    CONTINUE
 5300   CONTINUE
 5400  CONTINUE
 5500 CONTINUE
      WRITE(2,5505) 'N(I) FOR I = 1,2,3,4,5: ',N(1),N(2),N(3),N(4),N(5)
 5505 FORMAT(T3,A,5I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 6000 DO 6600 J1 = 1,N(1)
       DO 6500 J2 = 1,N(2)
        DO 6400 J3 = 1,N(3)
         DO 6300 J4 = 1,N(4)
          DO 6200 J5 = 1,N(5)
           DO 6100 J6 = 1,N(6)
            AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)*F(5,J5)*F(6,J6)
            AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)*W(5,J5)*W(6,J6)
            AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
            AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
            IF(AP.LT.AOP) FEP = FEP+AP
            IF(AX.GT.XOS) EXP = EXP+AP
            IF(AZ.GT.ZOS) EZP = EZP+AP
            TOT = TOT+AP
            TOP = TOP+AF
 6100      CONTINUE
 6200     CONTINUE
 6300    CONTINUE
 6400   CONTINUE
 6500  CONTINUE
 6600 CONTINUE
      WRITE(2,6605) 'N(I) FOR I = 1,...,6: ',N(1),N(2),N(3),N(4),N(5),
     &   N(6)
 6605 FORMAT(T3,A,6I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 7000 DO 7700 J1 = 1,N(1)
       DO 7600 J2 = 1,N(2)
        DO 7500 J3 = 1,N(3)
         DO 7400 J4 = 1,N(4)
          DO 7300 J5 = 1,N(5)
           DO 7200 J6 = 1,N(6)
            DO 7100 J7 = 1,N(7)
             AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)*F(5,J5)*F(6,J6)
     &           *F(7,J7)
             AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)*W(5,J5)*W(6,J6)
     &           *W(7,J7)
             AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &                   +X(7,J7)
             AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &                   +Z(7,J7)
             IF(AP.LT.AOP) FEP = FEP+AP
             IF(AX.GT.XOS) EXP = EXP+AP
             IF(AZ.GT.ZOS) EZP = EZP+AP
             TOT = TOT+AP
             TOP = TOP+AF
 7100       CONTINUE
 7200      CONTINUE
 7300     CONTINUE
 7400    CONTINUE
 7500   CONTINUE
 7600  CONTINUE
 7700 CONTINUE
      WRITE(2,7705) 'N(I) FOR I = 1,...,7: ',N(1),N(2),N(3),N(4),N(5),
     &   N(6),N(7)
 7705 FORMAT(T3,A,7I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 8000 DO 8800 J1 = 1,N(1)
       DO 8700 J2 = 1,N(2)
        DO 8600 J3 = 1,N(3)
         DO 8500 J4 = 1,N(4)
          DO 8400 J5 = 1,N(5)
           DO 8300 J6 = 1,N(6)
            DO 8200 J7 = 1,N(7)
             DO 8100 J8 = 1,N(8)
              AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)*F(5,J5)*F(6,J6)
     &            *F(7,J7)*F(8,J8)
              AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)*W(5,J5)*W(6,J6)
     &            *W(7,J7)*W(8,J8)
              AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &                    +X(7,J7)+X(8,J8)
              AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &                    +Z(7,J7)+Z(8,J8)
              IF(AP.LT.AOP) FEP = FEP+AP
              IF(AX.GT.XOS) EXP = EXP+AP
              IF(AZ.GT.ZOS) EZP = EZP+AP
              TOT = TOT+AP
              TOP = TOP+AF
 8100        CONTINUE
 8200       CONTINUE
 8300      CONTINUE
 8400     CONTINUE
 8500    CONTINUE
 8600   CONTINUE
 8700  CONTINUE
 8800 CONTINUE
      WRITE(2,8805) 'N(I) FOR I = 1,...,8: ',N(1),N(2),N(3),N(4),N(5),
     &   N(6),N(7),N(8)
 8805 FORMAT(T3,A,8I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
      GO TO 9910
 9000 DO 9900 J1 = 1,N(1)
       DO 9800 J2 = 1,N(2)
        DO 9700 J3 = 1,N(3)
         DO 9600 J4 = 1,N(4)
          DO 9500 J5 = 1,N(5)
           DO 9400 J6 = 1,N(6)
            DO 9300 J7 = 1,N(7)
             DO 9200 J8 = 1,N(8)
              DO 9100 J9 = 1,N(9)
               AF = F(1,J1)*F(2,J2)*F(3,J3)*F(4,J4)*F(5,J5)*F(6,J6)
     &             *F(7,J7)*F(8,J8)*F(9,J9)
               AP = AF*W(1,J1)*W(2,J2)*W(3,J3)*W(4,J4)*W(5,J5)*W(6,J6)
     &             *W(7,J7)*W(8,J8)*W(9,J9)
               AX = X(1,J1)+X(2,J2)+X(3,J3)+X(4,J4)+X(5,J5)+X(6,J6)
     &                     +X(7,J7)+X(8,J8)+X(9,J9)
               AZ = Z(1,J1)+Z(2,J2)+Z(3,J3)+Z(4,J4)+Z(5,J5)+Z(6,J6)
     &                     +Z(7,J7)+Z(8,J8)+Z(9,J9)
               IF(AP.LT.AOP) FEP = FEP+AP
               IF(AX.GT.XOS) EXP = EXP+AP
               IF(AZ.GT.ZOS) EZP = EZP+AP
               TOT = TOT+AP
               TOP = TOP+AF
 9100         CONTINUE
 9200        CONTINUE
 9300       CONTINUE
 9400      CONTINUE
 9500     CONTINUE
 9600    CONTINUE
 9700   CONTINUE
 9800  CONTINUE
 9900 CONTINUE
      WRITE(2,9905) 'N(I) FOR I = 1,...,9: ',N(1),N(2),N(3),N(4),N(5),
     &   N(6),N(7),N(8),N(9)
 9905 FORMAT(T3,A,9I5)
      FEP = FEP/TOT
      EXP = EXP/TOT
      EZP = EZP/TOT
 9910 WRITE(2,9920) 'COMBINED FISHER''S EXACT TEST P-VALUE = ',FEP
 9920 FORMAT(T3,A,D20.13)
      WRITE(2,9920) 'OBSERVED CHI-SQUARE TEST STATISTIC = ',XOS
      WRITE(2,9920) 'COMBINED EXACT CHI-SQUARE TEST P-VALUE = ',EXP
      WRITE(2,9920) 'OBSERVED LOG-LR TEST STATISTIC = ',ZOS
      WRITE(2,9920) 'COMBINED EXACT LOG-LR TEST P-VALUE = ',EZP
      WRITE(2,9920) 'TOTAL NUMBER OF COMBINED FREQUENCIES = ',TOT
      WRITE(2,9920) 'TOTAL NUMBER OF COMBINED CONFIGURATIONS = ',TOP
      END
