      PROGRAM M2
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM M2 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE FISHER'S EXACT PROBABILITY FOR
C     2 CELL GOODNESS-OF-FIT TABLES.
C
      DIMENSION  L(2),P(2),PL(2)
C
C     COMPUTE FISHER'S EXACT TEST FOR 2 CELL G-O-F TABLES
C
      WRITE(*,2) 'FISHER''S EXACT TEST FOR SPARSE 2 CELL G-O-F TABLES'
    2 FORMAT(/,T2,A,/)
      WRITE(*,3) 'ENTER L1 & L2:'
    3 FORMAT(T2,A)
      READ(*,*)  L(1),L(2)
      WRITE(*,3) 'ENTER P1:'
      READ(*,*)  P(1)
      P(2) = 1.0D0-P(1)
      N = 0
      DO 4 I = 1,2
           PL(I) = DLOG(P(I))
           N = N+L(I)
    4 CONTINUE
      WRITE(*,5)
    5 FORMAT(T15,'FISHER''S EXACT TEST FOR 2 CELL G-O-F TABLES',/)
      WRITE(*,6) '2 OBSERVED CELL VALUES: ',L(1),L(2)
    6 FORMAT(T5,A,2I4)
      WRITE(*,7) 'TOTAL OF ALL 2 CELL VALUES: ',N
    7 FORMAT(T5,A,I12)
      CALL FE2(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +         IFE,ICS,IZS,ILR,CHECK)
      WRITE(*,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE(*,7) '# OF TABLES YIELDING FISHER''S EXACT P-VALUE = ',IFE
      WRITE(*,8) 'P(OBSERVED TABLE) = ',PT
    8 FORMAT(T5,A,D16.9)
      WRITE(*,8) 'FISHER''S EXACT 2 CELL G-O-F TEST P-VALUE = ',PFE
      WRITE(*,7) '# OF TABLES YIELDING EXACT CHI-SQUARED P-VALUE = ',ICS
      IF(CST.LT.0.0D0) CST = 0.0D0
      WRITE(*,8) 'OBSERVED CHI-SQUARED STATISTIC = ',CST
      WRITE(*,8) 'EXACT CHI-SQUARED 2 CELL G-O-F TEST P-VALUE = ',PCS
      WRITE(*,7) '# OF TABLES YIELDING EXACT ZELTERMAN P-VALUE = ',IZS
      IF(CZT.LT.0.0D0) CZT = 0.0D0
      WRITE(*,8) 'OBSERVED ZELTERMAN STATISTIC = ',CZT
      WRITE(*,8) 'EXACT ZELTERMAN 2 CELL G-O-F TEST P-VALUE = ',PZS
      WRITE(*,7) '# OF TABLES YIELDING EXACT LR TEST P-VALUE = ',ILR
      IF(TLR.LT.0.0D0) TLR = 0.0D0
      WRITE(*,8) 'OBSERVED LR TEST STATISTIC = ',TLR
      WRITE(*,8) 'EXACT LR TEST 2 CELL G-O-F TEST P-VALUE = ',PLR
      WRITE(*,8) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE FE2(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +               IFE,ICS,IZS,ILR,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  A FISHER'S EXACT TEST FOR TESTING GOODNESS-OF-FIT FOR 2 CELLS.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PFE   -- OUTPUT, FISHER'S EXACT PROBABILITY VALUE (P-VALUE)
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        IFE   -- OUTPUT, # OF TABLES COMPRISING FISHER'S EXACT P-VALUE
C        CHECK -- OUTPUT, SUM OF ALL CONFIGURATION PROBABILITIES
C     USAGE:
C        SUBROUTINE FE2 COMPUTES THE FISHER EXACT PROBABILITY TEST
C        FOR THE GOODNESS-OF-FIT OF 2 OBSERVED CELL FREQUENCIES
C
      DIMENSION L(2),P(2),PL(2)
C
      Z = FAC(N)
C  DETERMINE THE PROBABILITY OF THE OBSERVED 2 CELL FREQUENCIES.
      THISP = PR(Z,PL,L)*(1.0D0-1.0D-12)
      CST = CS(N,P,L)*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CZT = CZ(N,P,L)*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      TLR = ZL(N,P,L)*(1.0D0-1.0D-12)
      IF(TLR.LT.1.0D-11) TLR = -1.0D-11
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      ITAB = 0
      IFE = 0
      ICS = 0
      IZS = 0
      ILR = 0
      PFE = 0.0D0
      PCS = 0.0D0
      PZS = 0.0D0
      PLR = 0.0D0
      CHECK = 0.0D0
C  1 MAJOR LOOP DETERMINES THE VALID TABLES FROM ALL VALUES OF 1 SPECIFIED
C  CELL (THE REMAINING CELL IS OBTAINED FROM THE 1 SPECIFIED CELL).
      DO 40 I1 = 0,N
      L(1) = I1
      L(2) = N-I1
C
       ITAB = ITAB+1
       CP = PR(Z,PL,L)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       IF(CP.LT.THISP) THEN
        IFE = IFE+1
        PFE = PFE+EXCP
       ENDIF
       CC = CS(N,P,L)
       IF(CC.GT.CST) THEN
        ICS = ICS+1
        PCS = PCS+EXCP
       ENDIF
       U = CZ(N,P,L)
       IF(U.GT.CZT) THEN
        IZS = IZS+1
        PZS = PZS+EXCP
       ENDIF
       W = ZL(N,P,L)
       IF(W.GT.TLR) THEN
        ILR = ILR+1
        PLR = PLR+EXCP
       ENDIF
   40 CONTINUE
      PT = DEXP(THISP)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION PR(Z,PL,L)
C  OBTAIN SUM OF LOG PROBABILITY FOR 2 CELLS.
      DOUBLE PRECISION  Z,FAC,PL(2)
      INTEGER  I,L(2)
      PR = Z
      DO 10 I = 1,2
         PR = PR-FAC(L(I))+L(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CS(N,P,L)
C  OBTAIN CHI-SQUARED STATISTIC
      DOUBLE PRECISION  ONE,P(2)
      INTEGER  I,L(2)
      ONE = 1.0D0
      CS = 0.0D0
      DO 10 I = 1,2
         CS = CS+(ONE*L(I))**2/P(I)
   10 CONTINUE
      CS = CS/N-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CZ(N,P,L)
C  OBTAIN ZELTERMAN STATISTIC
      DOUBLE PRECISION  ONE,P(2)
      INTEGER  I,L(2)
      ONE = 1.0D0
      CZ = 0.0D0
      DO 10 I = 1,2
         CZ = CZ+ONE*L(I)*(L(I)-1)/P(I)
   10 CONTINUE
      CZ = CZ/N+2-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION ZL(N,P,L)
C  OBTAIN LR TEST STATISTIC
      DOUBLE PRECISION  X,RZ,P(2)
      INTEGER  I,L(2)
      ZL = 0.0D0
      DO 10 I = 1,2
         X = 1.0D0*L(I)
         IF(X.GT.1.5D0) THEN
            RZ = X*DLOG(X)
         ELSE
            RZ = 0.0D0
         ENDIF
         ZL = ZL+RZ-DLOG(P(I)*N)*L(I)
   10 CONTINUE
      ZL = ZL*2
      RETURN
      END
