      PROGRAM RGF
      IMPLICIT REAL*8(A-H,O-Z)
C
C  THIS IS A RESAMPLING PROGRAM FOR GOODNESS-OF-FIT TESTS.  THE OBSERVED
C  FREQUENCIES AND PROBABILITIES FOR THE K DISCRETE CELLS ARE ENTERED
C  INTERACTIVELY (K < 51).  THE OUTPUT CONSISTS OF RESAMPLING P-VALUES
C  FOR GOODNESS-OF-FIT STATISTICS FOR THE FISHER EXACT, PEARSON CHI-SQUARE,
C  ZELTERMAN, AND LIKELIHOOD RATIO TESTS.  IN ADDITION TO THE OBSERVED
C  FREQUENCY AND PROBABILITY VALUES FOR EACH CELL, A POSITIVE INTEGER
C  SEED AND NUMBER OF SIMULATIONS ARE ALSO ENTERED.
C
C         PROGRAM MODIFIED 8/14/2003
C
      CHARACTER  FOUT*40
      DIMENSION  U(17),M(50),P(50),V(50)
C
C  IF K > 50, REPLACE 50 WITH K IN M(K), P(K) AND V(K) ABOVE.
C
      WRITE(*,*) 'ENTER K,M(1),...,M(K)'
      READ(*,*) K,(M(I),I=1,K)
      WRITE(*,*) 'ENTER (0,1) FOR (EQUAL,UNEQUAL) CELL PROBABILITIES:'
      READ(*,*) L
      IF(L.EQ.1) THEN
        WRITE(*,*) 'ENTER P(1),...,P(K-1)'
        READ(*,*) (P(I),I=1,K-1)
      ELSE
        P(1) = 1.0D0/DBLE(K)
        DO 1 I = 2,K-1
          P(I) = P(1)
    1   CONTINUE
      ENDIF
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,2) FOUT
    2 FORMAT(A)
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      ZERO = 0.0D0
      ONE = 1.0D0
      PHI = 1.0D-12
      N = 0
      DO 3 I = 1,K
        N = N+M(I)
    3 CONTINUE
      W = ZERO
      DO 4 I = 1,K-1
        IF(P(I).LE.ZERO)  STOP 'A P(I) IS NONPOSITIVE'
        W = W+P(I)
    4 CONTINUE
      IF(W.GE.ONE)  STOP 'P(K) IS NONPOSITIVE OR FAULTY P(I) VALUES'
      P(K) = ONE-W
      V(1) = P(1)
      DO 5 I = 2,K-1
         V(I) = V(I-1)+P(I)
    5 CONTINUE
      V(K) = ONE
      FN = FAC(N)
      T1 = ZERO
      T2 = ZERO
      T3 = ZERO
      T4 = ZERO
      DO 10 I=1,K
         T1 = T1+DLOG(P(I))*M(I)-FAC(M(I))
         T2 = T2+(P(I)*N-M(I))**2/(P(I)*N)
         T3 = T3+ONE*M(I)*(M(I)-1)/(P(I)*N)
         T4 = T4+RZ(M(I))-DLOG(P(I)*N)*M(I)
   10 CONTINUE
      TFE = (FN+T1)*(ONE-PHI)
      TCS = T2*(ONE-PHI)
      IF(TCS.LT.1.0D-11) TCS = -1.0D-11
      TZ = T3*(ONE-PHI)
      IF(TZ.LT.1.0D-11) TZ = -1.0D-11
      TLR = T4*(ONE-PHI)*2
      IF(TLR.LT.1.0D-11) TLR = -1.0D-11
C
      KK = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,KK
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
C
      LFE = 0
      LCS = 0
      LZ = 0
      LLR = 0
      DO 250 IW = 1,MS
         DO 200 I = 1,K
            M(I) = 0
  200    CONTINUE
         DO 210 I = 1,N
            Z = UNI(U)
            DO 205 J = 1,K
               IF(Z.LE.V(J)) THEN
                  M(J) = M(J)+1
                  GO TO 210
               END IF
  205       CONTINUE
  210    CONTINUE
         T1 = ZERO
         T2 = ZERO
         T3 = ZERO
         T4 = ZERO
         DO 220 I=1,K
            T1 = T1+DLOG(P(I))*M(I)-FAC(M(I))
            T2 = T2+(P(I)*N-M(I))**2/(P(I)*N)
            T3 = T3+ONE*M(I)*(M(I)-1)/(P(I)*N)
            T4 = T4+RZ(M(I))-DLOG(P(I)*N)*M(I)
  220    CONTINUE
         T1 = FN+T1
         IF(T1.LT.TFE) LFE = LFE+1
         IF(T2.GT.TCS) LCS = LCS+1
         IF(T3.GT.TZ) LZ = LZ+1
         T4 = T4*2
         IF(T4.GT.TLR) LLR = LLR+1
  250 CONTINUE
      PFE = ONE*LFE/MS
      PCS = ONE*LCS/MS
      PZ = ONE*LZ/MS
      PLR = ONE*LLR/MS
C
      OPEN(5,FILE=FOUT,STATUS='NEW')
      WRITE(5,315) 'RESAMPLING DISCRETE GOODNESS-OF-FIT TESTS ',
     +             'WITH ',K,' CELLS AND ',N,' CASES'
      TFE = DEXP(TFE)
      WRITE(5,320) TFE, ' = FISHER EXACT G-O-F TEST STATISTIC'
      WRITE(5,320) PFE, ' = RESAMPLING FISHER EXACT TEST P-VALUE'
      IF(TCS.LT.ZERO) TCS = ZERO
      WRITE(5,320) TCS, ' = CHI-SQUARED G-O-F TEST STATISTIC'
      WRITE(5,320) PCS, ' = RESAMPLING CHI-SQUARED TEST P-VALUE'
      WRITE(5,325) K-1, ' = DEGREES OF FREEDOM FOR ASYMPTOTIC TESTS'
      IF(TCS.GT.ZERO) THEN
         CALL CHISQ(TCS,K-1,APCS)
      ELSE
         APCS = ONE
      ENDIF
      WRITE(5,320) APCS, ' = ASYMPTOTIC CHI-SQUARED TEST P-VALUE'
      IF(TZ.LT.ZERO) TZ = ZERO
      WRITE(5,320) TZ, ' = ZELTERMAN G-O-F TEST STATISTIC'
      WRITE(5,320) PZ, ' = RESAMPLING ZELTERMAN TEST P-VALUE'
      IF(TZ.GT.ONE*(N-K)) THEN
         CALL CHISQ(TZ+K-N,K-1,APZ)
      ELSE
         APZ = ONE
      ENDIF
      WRITE(5,320) APZ, ' = ASYMPTOTIC ZELTERMAN TEST P-VALUE'
      IF(TLR.LT.ZERO) TLR = ZERO
      WRITE(5,320) TLR, ' = LIKELIHOOD-RATIO G-O-F TEST STATISTIC'
      WRITE(5,320) PLR, ' = RESAMPLING LIKELIHOOD-RATIO TEST P-VALUE'
      IF(TLR.GT.ZERO) THEN
         CALL CHISQ(TLR,K-1,APLR)
      ELSE
         APLR = ONE
      ENDIF
      WRITE(5,320) APLR, ' = ASYMPTOTIC LIKELIHOOD-RATIO TEST P-VALUE'
      WRITE(5,330) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
      WRITE(5,330) 'NUMBER OF SIMULATIONS = ',MS
  315 FORMAT(/,T2,A,A,I3,A,I9,A,/)
  320 FORMAT(T5,D16.9,A)
  325 FORMAT(T5,I4,A)
  330 FORMAT(T5,A,I9)
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION RZ(I)
C  FIND I(LOG(I))
      DOUBLE PRECISION  X
      X = 1.0D0*I
      IF(X.GT.1.5D0) THEN
         RZ = X*DLOG(X)
      ELSE
         RZ = 0.0D0
      ENDIF
      RETURN
      END
C...........................................................................
      SUBROUTINE CHISQ(T,N,P)
      DOUBLE PRECISION T,ZERO,ONE,V,X,P,PI,PX,A,B,E1,E2,E3,E4,E5,H,W
      ZERO = 0.0D0
      ONE = 1.0D0
      IF(T.LE.ZERO) THEN
         P = ONE
         GO TO 60
      END IF
      IF(N.EQ.1) THEN
         X = DSQRT(T)
         CALL PVN(X,PX)
         P = PX*2
         GO TO 60
      END IF
      V = ONE*N
      IF(V/2-N/2.GT.0.1D0) GO TO 30
      P = DEXP(-T/2)
      IF(N.LT.3) GO TO 60
      K = (N-2)/2
      B = ONE
      DO 20 J = 1,K
         L = J
         A = ONE
         DO 10 I = 1,L
            A = A*T/(2*I)
   10    CONTINUE
         B = B+A
   20 CONTINUE
      P = P*B
      GO TO 60
   30 PI = DATAN(ONE)*4
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*DSQRT(T)+1)
      P = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(-T/2)/DSQRT(PI/2)
      IF(N.LT.2) GO TO 60
      K = (N-1)/2
      B = ZERO
      DO 50 J = 1,K
         L = J
         A = ONE
         DO 40 I = 1,L
            A = A*T/(2*I-1)
   40    CONTINUE
         B = B+A
   50 CONTINUE
      P = P+B*DEXP(-T/2)/DSQRT(PI*T/2)
   60 RETURN
      END
C
C
      SUBROUTINE PVN(Z,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,G,A,B,C,EE,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*EE(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = EE(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION EE(G)
C
      DOUBLE PRECISION G
C
      EE = 0.0D0
      IF(G.GE.-709.1D0) EE = DEXP(G)
      RETURN
      END
