      PROGRAM RTC4
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM RTC4 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE EXACT TREATMENT-COMPARED-TO-CONTROL
C     RIDIT ANALYSIS LOWER AND UPPER TAIL P-VALUES FOR A
C     4 CELL TABLE.
C
      DIMENSION  K(4),L(4),PL(4),R(4)
      CHARACTER  FOUT*40
C
      WRITE(*,*)  'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,1) FOUT
    1 FORMAT(A)
      ONE = 1.0D0
      WRITE(*,2) 'TREATMENT-CONTROL RIDIT ANALYSIS FOR 4 CELL TABLES'
    2 FORMAT(T2,A)
      WRITE(*,3) 'ENTER CONTROL FREQUENCIES K1, K2, K3 & K4:'
    3 FORMAT(T2,A)
      READ(*,*)  K(1),K(2),K(3),K(4)
      WRITE(*,3) 'ENTER TREATMENT FREQUENCIES L1, L2, L3 & L4:'
      READ(*,*)  L(1),L(2),L(3),L(4)
      NC = 0
      NT = 0
      DO 4 I = 1,4
           NC = NC+K(I)
           NT = NT+L(I)
    4 CONTINUE
      R(1) = ONE*K(1)/(2*NC)
      R(2) = ONE*K(1)/NC+ONE*K(2)/(2*NC)
      R(3) = ONE*(K(1)+K(2))/NC+ONE*K(3)/(2*NC)
      R(4) = ONE*(K(1)+K(2)+K(3))/NC+ONE*K(4)/(2*NC)
      XC = 0.0D0
      XT = 0.0D0
      DO 5 I = 1,4
           PL(I) = DLOG(0.25D0)
           XC = XC+R(I)*K(I)
           XT = XT+R(I)*L(I)
    5 CONTINUE
      XC = XC/NC
      XT = XT/NT
      OPEN(2,FILE=FOUT,STATUS='NEW')
      WRITE(2,6)
    6 FORMAT(T10,'EXACT TREATMENT-CONTROL RIDIT ANALYSIS FOR ',
     +           '4 CELL TABLES')
      WRITE(2,7) '4 CONTROL CELL FREQUENCIES: ',K(1),K(2),K(3),K(4)
    7 FORMAT(T5,A,4I8)
      WRITE(2,7) '4 TREATMENT CELL FREQUENCIES: ',L(1),L(2),L(3),L(4)
      WRITE(2,8) 'TOTAL OF ALL 4 CONTROL CELL FREQUENCIES: ',NC
    8 FORMAT(T5,A,I12)
      WRITE(2,8) 'TOTAL OF ALL 4 TREATMENT CELL FREQUENCIES: ',NT
      CALL ER4(L,PL,R,NT,PT,XT,PERL,PERU,ITAB,IERL,IERU,
     +     CHECK,Y1,Y2)
      SE = DSQRT(Y2-Y1**2)
      W = (XT-Y1)/SE
      CALL NORMAL(W,PNL,PNU)
      WRITE(2,8) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE(2,8) '# OF TABLES IN EXACT LOWER TAIL P-VALUE = ',IERL
      WRITE(2,8) '# OF TABLES IN EXACT UPPER TAIL P-VALUE = ',IERU
      WRITE(2,9) 'P(OBSERVED TABLE) = ',PT
    9 FORMAT(T5,A,D16.9)
      WRITE(2,9) 'CHECK THAT CONTROL RIDIT STATISTIC IS 0.5: ',XC
      WRITE(2,9) 'OBSERVED TREATMENT RIDIT STATISTIC = ',XT
      WRITE(2,9) 'EXACT EXPECTED VALUE OF RIDIT STATISTIC = ',Y1
      WRITE(2,9) 'EXACT STANDARD ERROR OF RIDIT STATISTIC = ',SE
      WRITE(2,9) 'EXACT STANDARDIZED NORMAL TEST STATISTIC = ',W
      WRITE(2,9) 'NORMAL LOWER TAIL P-VALUE FOR 4 CELL TABLE = ',PNL
      WRITE(2,9) 'NORMAL UPPER TAIL P-VALUE FOR 4 CELL TABLE = ',PNU
      WRITE(2,9) 'EXACT LOWER TAIL P-VALUE FOR 4 CELL TABLE = ',PERL
      WRITE(2,9) 'EXACT UPPER TAIL P-VALUE FOR 4 CELL TABLE = ',PERU
      WRITE(2,9) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE ER4(L,PL,R,N,PT,W,PERL,PERU,ITAB,IERL,IERU,
     +           CHECK,Y1,Y2)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  EXACT TEST FOR TREATMENT-COMPARED-TO-CONTROL RIDIT ANALYSIS OF 4 CELL TABLE.
C
C  DESCRIPTION OF VARIABLES IN CALL LIST:
C     L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C     PL    -- INPUT,  ARRAY CONTAINING THE CELL PROBABILITY LOGARITHMS
C     R     -- INPUT,  ARRAY CONTAINING OBSERVED CONTROL RIDITS
C     PERL  -- OUTPUT, EXACT RIDIT ANALYSIS LOWER-TAIL P-VALUE
C     PERU  -- OUTPUT, EXACT RIDIT ANALYSIS UPPER-TAIL P-VALUE
C     PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C     ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C     IERL  -- OUTPUT, # OF TABLES COMPRISING RIDIT ANALYSIS LOWER-TAIL P-VALUE
C     IERU  -- OUTPUT, # OF TABLES COMPRISING RIDIT ANALYSIS UPPER-TAIL P-VALUE
C     CHECK -- OUTPUT, SUM OF ALL CONFIGURATION PROBABILITIES
C  USAGE:
C     SUBROUTINE ER4 COMPUTES THE RIDIT ANALYSIS P-VALUES
C     FOR TESTING ORDERED PECULIARITY OF A 4 CELL TABLE
C
      DIMENSION L(4),PL(4),R(4)
C
      Z = FAC(N)
      PT = DEXP(PR(Z,PL,L))
C  DETERMINE THE PROBABILITY OF THE OBSERVED 4 CELL FREQUENCIES.
      XL = W*(1.0D0+1.0D-12)
      XU = W*(1.0D0-1.0D-12)
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      ITAB = 0
      IERL = 0
      IERU = 0
      PERL = 0.0D0
      PERU = 0.0D0
      CHECK = 0.0D0
      Y1 = 0.0D0
      Y2 = 0.0D0
C  3 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 3 SPECIFIED
C  CELLS (THE REMAINING CELL IS OBTAINED FROM THE 3 SPECIFIED CELLS).
      DO 40 I1 = 0,N
      M2 = N-I1
      DO 39 I2 = 0,M2
      M3 = N-I1-I2
      DO 38 I3 = 0,M3
       L(1) = I1
       L(2) = I2
       L(3) = I3
       L(4) = N-I1-I2-I3
       X = 0.0D0
C
       ITAB = ITAB+1
       CP = PR(Z,PL,L)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       DO 10 I = 1,4
           X = X+R(I)*L(I)
   10  CONTINUE
       X = X/N
       Y1 = Y1+EXCP*X
       Y2 = Y2+EXCP*X**2
       IF(X.LT.XL) THEN
        IERL = IERL+1
        PERL = PERL+EXCP
       ENDIF
       IF(X.GT.XU) THEN
        IERU = IERU+1
        PERU = PERU+EXCP
       ENDIF
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION PR(Z,PL,L)
C  OBTAIN SUM OF LOG PROBABILITY FOR 4 CELLS.
      DOUBLE PRECISION  Z,FAC,PL(4)
      INTEGER  I,L(4)
      PR = Z
      DO 10 I = 1,4
         PR = PR-FAC(L(I))+L(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C  ...........................................................................
      SUBROUTINE NORMAL(Z,PNL,PNU)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PNL,PNU,PROB,ZERO,ONE,PI,G,A,B,C,E,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.GT.ZERO) THEN
         PNU = PROB
         PNL = ONE-PROB
      ELSE
         PNL = PROB
         PNU = ONE-PROB
      END IF
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
